*****************************************************************************************
*                                                                                       *
*  AppleIIgsKarate.AI.s :  Artificial Intelligence functions for objects on screen.     *
*                                                                                       *
*****************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI  *  Brutal Deluxe Software  *  July 2022  *
*****************************************************************************************

************************************************************************
*  ShurikenAIMove() :  Shuriken Sprite AI Move Function (IN:X=Sprite). *
************************************************************************
ShurikenAIMove      STX        SAIM_SPRITE_INDEX    ; Compute Shuriken next move
*--
SAIM_SpriteID       LDA        SE_SPRITE_ID,X       ; Invert Sprite ID 89<->8A
                    CMP        #$0089
                    BEQ        SAIM_SpriteID_1
                    LDA        #$0089
                    STA        SAIM_SPRITE_ID
                    BRA        SAIM_SpriteDir
SAIM_SpriteID_1     LDA        #$008A
                    STA        SAIM_SPRITE_ID     
*--
SAIM_SpriteDir      LDA        SE_SPRITE_DATA_1,X   ; Sprite Direction (0:right,1:left)
                    CMP        #$0001
                    BEQ        SAIM_Left
*--
SAIM_Right          LDA        SE_SPRITE_X0,X       ; >>> Shuriken goes right
                    CMP        #$009C               ; 314 (right border)
                    BGE        SAIM_Zombi       
                    LDA        SE_SPRITE_X0,X
                    INC
                    INC                             ; X0 += 2
                    XBA
                    CLC
                    ADC        SE_SPRITE_Y0,X
                    TAY                             ; Y = SpriteX0|SpriteY0
                    LDA        SAIM_SPRITE_ID
                    JSR        InstallNextSprite
                    BRA        SAIM_End
*--
SAIM_Left           LDA        SE_SPRITE_X0,X       ; <<< Shuriken goes left
                    CMP        #$0002               ; 1 (left border)
                    BLT        SAIM_Zombi       
                    LDA        SE_SPRITE_X0,X
                    DEC
                    DEC                             ; X0 -= 2
                    XBA
                    CLC
                    ADC        SE_SPRITE_Y0,X
                    TAY                             ; Y = SpriteX0|SpriteY0
                    LDA        SAIM_SPRITE_ID
                    JSR        InstallNextSprite    ; Backup + Install
                    BRA        SAIM_End                    
*--
SAIM_Zombi          JSR        SetSpriteZombi       ; Turn Sprite as Zombi
                    JSR        BackupSpriteInfo     ; Don't move Sprite position
*--
SAIM_End            LDX        SAIM_SPRITE_INDEX
                    RTS

*----------   ShurikenAIMove Variables   ---------
SAIM_SPRITE_INDEX   HEX        0000                 ; Current Sprite Index (0,2,4,6...)
SAIM_SPRITE_ID      HEX        0000


************************************************************************
*  ShurikenAISend() :  Shuriken Sprite AI Send Function (IN:X=Sprite). *
************************************************************************
ShurikenAISend      STX        SAIS_SPRITE_INDEX    ; React on collision with Blue Player
*--
SAIS_Check_Hit      TXA                             ; Check if the Shuriken has hit the Blue Player
                    STA        SAIS_Check_Hit_1+1
                    ASL
                    CLC
SAIS_Check_Hit_1    ADC        #$AAAA               ; Suriken Index(*2)*3
                    LDX        #$000E               ; Blue Player Index
                    CLC
                    ADC        SE_SPRITE_RECV,X     ; Blue Player Receive Table
                    TAY                             ; Shuriken Line @ into Blue Player Send Table                    
                    LDA:       $0000,Y              ; Score Shuriken into Blue Player
                    BEQ        SAIS_End
*--
SAIS_Touch          LDA:       $0004,Y              ; Impact X0|Y0 Address on Screen
                    JSR        AddImpactSprite      ; Create Impact Sprite
                    LDX        #$000E               ; Blue Player Index
                    LDA        #$0020               ; Decrease Blue Player Life Bar
                    JSR        DecrementLifeBar     
                    LDA        #$0009               ; Add Touched Sound
                    JSR        AppendToSoundList                   
                    LDX        SAIS_SPRITE_INDEX    ; Delete Shuriken Sprite
                    BRA        SAIS_Zombi     
*--
SAIS_Zombi          JSR        SetSpriteZombi       ; Turn Sprite as Zombi
                    JSR        BackupSpriteInfo     ; Don't move Sprite position
*--
SAIS_End            LDX        SAIS_SPRITE_INDEX
                    RTS

*----------   ShurikenAISend Variables   ---------
SAIS_SPRITE_INDEX   HEX        0000                 ; Current Sprite Index (0,2,4,6...)
SAIS_SPRITE_ID      HEX        0000


***************************************************************************
*  BonusVaseAIMove() :  Bonus Vase Sprite Move AI Function (IN:X=Sprite). *
***************************************************************************
BonusVaseAIMove     STX        BVAIM_SPRITE_INDEX   ; Compute Bonus Vase next move
*------
BVAIM_Status        LDA        SE_SPRITE_DATA_3,X   ; Check Vase Status (0=Bouncing,1=Breaking,2=BreakingInit)
                    BEQ        BVAIM_Bouncing
                    CMP        #$0001
                    BEQ        BVAIM_Breaking
*------
BVAIM_BreakingInit  LDA        #$0001               ; Change Status to Breaking
                    STA        SE_SPRITE_DATA_3,X 
                    BRA        BVAIM_End            ; New Sprite has been installed in Received AI function
*------
BVAIM_Breaking      LDA        SE_SPRITE_DATA_1,X   ; Running the Break Sequence
                    INC
                    INC
                    CMP        #$0006               ; End of Sequence (3*2)
                    BNE        BVAIM_Breaking_1
                    JMP        BVAIM_Zombi
BVAIM_Breaking_1    STA        SE_SPRITE_DATA_1,X   ; Next Steps : (00 - 03)*2
                    TAY
                    LDA        BVAIM_BROKEN_ID,Y
                    STA        BVAIM_BreakSpriteID+1
                    LDA        SE_SPRITE_DATA_2,X   ; SpriteX0|SpriteY0
                    AND        #$FF00               ; X0|00
                    XBA
                    CLC
                    ADC        BVAIM_BROKEN_DELTA_X,Y
                    XBA
                    ORA        SE_SPRITE_Y0,X       ; 00|Y0
                    CLC
                    ADC        BVAIM_BROKEN_DELTA_Y,Y
                    TAY                             ; Y = SpriteX0|SpriteY0
BVAIM_BreakSpriteID LDA        #$AAAA               ; Sprite ID = Smoke (Big to Small)
                    JSR        InstallNextSprite    ; Backup + Install
                    JMP        BVAIM_End
*------
BVAIM_Bouncing      LDA        SE_SPRITE_DATA_1,X   ; Sprite Sequence Number 2*(0-N)
                    INC
                    INC
                    CMP        #$0080               ; End of Sequence (64*2)
                    BEQ        BVAIM_Zombi
*--
                    STA        SE_SPRITE_DATA_1,X   ; Sprite Sequence
                    TAY
                    LDA        BVAIM_SPRITE_ID,Y    ; Sprite ID
                    STA        BVAIM_SpriteID+1
                    LDA        SE_SPRITE_DATA_2,X   ; Sprite X0Y0 base
                    AND        #$FF00
                    ORA        BVAIM_SPRITE_Y0,Y
                    XBA
                    CLC
                    ADC        BVAIM_SPRITE_DELTAX0,Y
                    XBA
                    TAY                             ; Y = SpriteX0|SpriteY0
BVAIM_SpriteID      LDA        #$AAAA               ; Sprite ID
                    JSR        InstallNextSprite    ; Backup + Install
                    BRA        BVAIM_End
*----
BVAIM_Zombi         JSR        SetSpriteZombi       ; Turn Sprite as Zombi
                    JSR        BackupSpriteInfo     ; Don't move Sprite position
*----
BVAIM_End           LDX        BVAIM_SPRITE_INDEX
                    RTS

*----------   BonusVaseAIMove Variables   ---------
BVAIM_SPRITE_INDEX  HEX        0000                 ; Current Sprite Index (0,2,4,6...)
BVAIM_SPRITE_ID     HEX        8100,8100,8200,8300,8400                ; Smoke : Small to Big
                    HEX        4000,4000,4000,4000,4000,4000,4000,4000 ; Vase
                    HEX        4000,4000,4000,4000,4000,4000,4000,4000
                    HEX        4000,4000,4000,4000,4000,4000,4000,4000
                    HEX        4000,4000,4000,4000,4000,4000,4000,4000
                    HEX        4000,4000,4000,4000,4000,4000,4000,4000
                    HEX        4000,4000,4000,4000,4000,4000,4000,4000
                    HEX        4000,4000,4000,4000,4000,4000,4000,4000
                    HEX        8400,8300,8200                          ; Smoke : Big to Small
BVAIM_SPRITE_DELTAX0 HEX       0200,0200,0100,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0000,0000,0000,0000,0000,0000
                    HEX        0000,0000,0100
BVAIM_SPRITE_Y0     HEX        8C00,8C00,8A00,8900,8700
                    HEX        8800,8900,8A00,8C00,8F00,9400,9C00,A800 ; Go Down
                    HEX        9C00,9400,8F00,8C00,8A00,8900,8800,8700 ; Go Up
                    HEX        8800,8900,8A00,8C00,8F00,9400,9C00,A800 ; Go Down
                    HEX        9C00,9400,8F00,8C00,8A00,8900,8800,8700 ; Go Up
                    HEX        8800,8900,8A00,8C00,8F00,9400,9C00,A800 ; Go Down
                    HEX        9C00,9400,8F00,8C00,8A00,8900,8800,8700 ; Go Up
                    HEX        8800,8900,8A00,8C00,8F00,9400,9C00,A800 ; Go Down
                    HEX        A800,AA00,AA00
BVAIM_BROKEN_ID     HEX        8400,8300,8200       ; Smoke : Big to Small
BVAIM_BROKEN_DELTA_X HEX       0000,0000,0100
BVAIM_BROKEN_DELTA_Y HEX       0000,0200,0000


*********************************************************************************
*  BonusVaseAIReceive() :  Bonus Vase Sprite Receive AI Function (IN:X=Sprite). *
*********************************************************************************
BonusVaseAIReceive  STX        BVAIR_SPRITE_INDEX   ; React to Collision with Blue Player
*------
BVAIR_Status        LDA        SE_SPRITE_DATA_3,X   ; Check Vase Status (0=bouncing,1=breaking)
                    BNE        BVAIR_End
*------
BVAIR_Check_Hit     LDA        SE_SPRITE_RECV,X     ; Check if the Vase has been hit by the Blue Player
                    CLC
                    ADC        #$002A               ; Blue Player Offset (#7*2*3)
                    STA        BVAIR_BluePlayer+1   ; Blue Player Collision Line       
                    LDA        SE_MAX_SPRITE_NB     
                    ASL
                    STA        BVAIR_Check_Hit_1+1
                    ASL                             
BVAIR_Check_Hit_1   ADC        #$AAAA               ; Table Size (NbSprite*6)
                    ADC        SE_SPRITE_RECV,X     ; Table Begin @
                    STA        BVAIR_Next_Limit+1   ; Table End @
                    LDA        SE_SPRITE_RECV,X     
                    TAY
BVAIR_Loop          LDA:       $0000,Y              ; Score
                    BEQ        BVAIR_Continue
BVAIR_BluePlayer    CPY        #$AAAA               ; Check if this is the Blue Player
                    BEQ        BVAIR_Broken
BVAIR_Continue      TYA
                    CLC
                    ADC        #$0006               ; Entry Size
                    TAY
BVAIR_Next_Limit    CPY        #$AAAA               ; End of Table
                    BNE        BVAIR_Loop
                    BRA        BVAIR_End            ; Not Hit
*------
BVAIR_Broken        LDA        #$0001               ; Break Vase Sound
                    JSR        AppendToSoundList    ; Add this Sound ID to the list
                    LDA        #$0005               ; Increment User Score +500
                    JSR        IncrementScore
                    JSR        FillUpBlueLifeBar    ; Full Life Bar
                    LDX        BVAIR_SPRITE_INDEX   
                    LDA        #$0002               ; Change Vase Status = Breaking Init
                    STA        SE_SPRITE_DATA_3,X   
                    STZ        SE_SPRITE_DATA_1,X   ; Breaking Animation Sequence Number 2*(0-N)
                    LDA        SE_SPRITE_DATA_2,X   ; SpriteX0|SpriteY0
                    AND        #$FF00               ; X0|00
                    ORA        SE_SPRITE_Y0,X       ; 00|Y0                                     
                    TAY                             ; Y = SpriteX0|SpriteY0
                    LDA        #$0084               ; Sprite ID = Big Smoke
                    JSR        InstallNextSprite    ; Backup + Install
*----
BVAIR_End           LDX        BVAIR_SPRITE_INDEX
                    RTS

*----------   BonusVaseAIReceive Variables   ---------
BVAIR_SPRITE_INDEX  HEX        0000                 ; Current Sprite Index (0,2,4,6...)


********************************************************************************
*  LevelVaseAIMove() :  Levels 4|8 Vase Sprite AI Move Function (IN:X=Sprite). *
********************************************************************************
LevelVaseAIMove     STX        LVAIM_SPRITE_INDEX   ; Compute Level Vase next move
*------
LVAIM_Status        LDA        SE_SPRITE_DATA_3,X   ; Check Vase Status (0=Bouncing,1=Breaking,2=BreakingInit)
                    BEQ        LVAIM_Bouncing
                    CMP        #$0001
                    BEQ        LVAIM_Breaking
*------
LVAIM_BreakingInit  LDA        #$0001               ; Change Status to Breaking
                    STA        SE_SPRITE_DATA_3,X 
                    BRA        LVAIM_End            ; New Sprite has already been installed in Received AI function
*------
LVAIM_Breaking      LDA        SE_SPRITE_DATA_1,X   ; Running the Break Sequence
                    INC
                    INC
                    CMP        #$0006               ; End of Sequence (3*2)
                    BEQ        LVAIM_Zombi
                    STA        SE_SPRITE_DATA_1,X   ; Next Steps : (00 - 03)*2
                    TAY
                    LDA        LVAIM_BROKEN_ID,Y
                    STA        LVAIM_SpriteID+1
                    LDA        SE_SPRITE_DATA_2,X   ; SpriteX0|SpriteY0
                    AND        #$FF00               ; X0|00
                    XBA
                    CLC
                    ADC        LVAIM_BROKEN_DELTA_X,Y
                    XBA
                    ORA        SE_SPRITE_Y0,X       ; 00|Y0
                    CLC
                    ADC        LVAIM_BROKEN_DELTA_Y,Y
                    TAY                             ; Y = SpriteX0|SpriteY0
LVAIM_SpriteID      LDA        #$AAAA               ; Sprite ID = Smoke (Big to Small)
                    JSR        InstallNextSprite    ; Backup + Install
                    BRA        LVAIM_End
*------
LVAIM_Zombi         JSR        SetSpriteZombi       ; Turn Sprite as Zombi
                    JSR        BackupSpriteInfo     ; Don't move Sprite position
                    BRA        LVAIM_End
*------
LVAIM_Bouncing      LDX        LVAIM_SPRITE_INDEX   ; Running the Bouncing Sequence
                    LDA        SE_SPRITE_DATA_1,X   ; Sprite Sequence Number 2*(0-N)
                    INC
                    INC
                    CMP        #$0020               ; End of Sequence (16*2)
                    BNE        LVAIM_Steps
                    LDA        #$0000               ; First Step
LVAIM_Steps         STA        SE_SPRITE_DATA_1,X   ; Next Steps : (00 - 16)*2
*--
                    TAY
                    LDA        SE_SPRITE_DATA_2,X   ; SpriteX0|SpriteY0
                    AND        #$FF00               ; X0|00
                    ORA        LVAIM_SPRITE_Y0,Y    ; 00|Y0
                    TAY                             ; Y = SpriteX0|SpriteY0
                    LDA        #$0040               ; Sprite ID = Vase
                    JSR        InstallNextSprite    ; Backup + Install
*--
LVAIM_End           LDX        LVAIM_SPRITE_INDEX
                    RTS

*----------   LevelVaseAIMove Variables   ---------
LVAIM_SPRITE_INDEX  HEX        0000                 ; Current Sprite Index (0,2,4,6...)
LVAIM_SEND_OFFSET   HEX        0000                 ; Index*3 (0,6,12,...)
LVAIM_SPRITE_Y0     HEX        8800,8900,8A00,8C00,8F00,9400,9C00,A800  ; Go Down
                    HEX        9C00,9400,8F00,8C00,8A00,8900,8800,8700  ; Go Up
LVAIM_BROKEN_ID     HEX        8400,8300,8200       ; Smoke : Big to Small
LVAIM_BROKEN_DELTA_X HEX       0000,0000,0100
LVAIM_BROKEN_DELTA_Y HEX       0000,0200,0000


**************************************************************************************
*  LevelVaseAIReceive() :  Levels 4|8 Vase Sprite AI Receive Function (IN:X=Sprite). *
**************************************************************************************
LevelVaseAIReceive  STX        LVAIR_SPRITE_INDEX   ; React to Collision with Blue Player
*------
LVAIR_Status        LDA        SE_SPRITE_DATA_3,X   ; Check Vase Status (0=bouncing,1=breaking)
                    BNE        LVAIR_End
*------
LVAIR_Check_Hit     LDA        SE_MAX_SPRITE_NB     ; Check if the Vase has been hit by the Player
                    ASL
                    STA        LVAIR_Check_Hit_1+1
                    ASL
                    CLC
LVAIR_Check_Hit_1   ADC        #$AAAA               ; Table Size (NbSprite*6)
                    ADC        SE_SPRITE_RECV,X     ; Table Start @
                    STA        LVAIR_Next_Limit+1   ; Table End @
                    LDA        SE_SPRITE_RECV,X     
                    TAY
LVAIR_Loop          LDA:       $0000,Y              ; Score
                    BNE        LVAIR_Broken
                    TYA
                    CLC
                    ADC        #$0006               ; Entry Size
                    TAY
LVAIR_Next_Limit    CPY        #$AAAA               ; End of Table
                    BNE        LVAIR_Loop
                    BRA        LVAIR_End            ; Not Hit
*------
LVAIR_Broken        LDA        #$0001               ; Break Vase Sound
                    JSR        AppendToSoundList    ; Add this Sound ID to the list
                    LDA        #$0001               ; Increment User Score +100
                    JSR        IncrementScore
                    LDX        LVAIR_SPRITE_INDEX   
                    LDA        #$0002
                    STA        SE_SPRITE_DATA_3,X   ; Change Vase Status = BreakingSkip
                    STZ        SE_SPRITE_DATA_1,X   ; Breaking Animation Sequence Number 2*(0-N)
                    LDA        SE_SPRITE_DATA_2,X   ; SpriteX0|SpriteY0
                    AND        #$FF00               ; X0|00
                    ORA        SE_SPRITE_Y0,X       ; 00|Y0                                     
                    TAY                             ; Y = SpriteX0|SpriteY0
                    LDA        #$0084               ; Sprite ID = Big Smoke
                    JSR        InstallNextSprite    ; Backup + Install
*------
LVAIR_End           LDX        LVAIR_SPRITE_INDEX
                    RTS

*----------   LevelVaseAIReceive Variables   ---------
LVAIR_SPRITE_INDEX  HEX        0000                 ; Current Sprite Index (0,2,4,6...)


*************************************************************************
*  ImpactAIMove() :  Hit Impact Sprites Move IA function (IN:X=Sprite). *
*************************************************************************
ImpactAIMove        PHX                             ; Display Impact Sprites
*--
ImpactAIM_Sequence  LDA        SE_SPRITE_DATA_1,X   ; Sequence #
                    INC
                    INC
                    STA        SE_SPRITE_DATA_1,X
                    CMP        #$0008               ; End of Sequence
                    BEQ        ImpactAIM_Zombi
                    LDA        SE_SPRITE_DATA_2,X   ; Y=SpriteX0|SpriteY0
                    TAY
                    LDA        SE_SPRITE_ID,X       ; A=Sprite ID++ (85-88)
                    INC
                    JSR        InstallNextSprite    ; Backup + Install
                    BRA        ImpactAIM_End
*--                    
ImpactAIM_Zombi     JSR        SetSpriteZombi       ; Turn Sprite as Zombi
                    JSR        BackupSpriteInfo     ; Don't move Sprite position
*--
ImpactAIM_End       PLX
                    RTS


****************************************************************************
*  PlayerBlueAIMove() :  Human Player Blue IA Move function (IN:X=Sprite). *
****************************************************************************
PlayerBlueAIMove    PHX                             ; User Player Blue Moves
*--
                    JSR        BackupSpriteInfo     ; Backup Sprite
*--
PBAIM_ExtraSequence LDA        SE_SPRITE_NEXT_SEQ,X ; Any Sequence pending ?
                    BEQ        PBAIM_ApplyInput
                    CMP        #$FFFF               ; Blue player has died
                    BEQ        PBAIM_Death
                    AND        #$8000               ; Blue User player has been hurted             
                    BNE        PBAIM_Hurted                    
                    BRA        PBAIM_ES_End
*-
PBAIM_Hurted        LDA        SE_SPRITE_NEXT_SEQ,X ; Hurted Sequence ID (from Stand-Up)
                    AND        #$7FFF               ; Clear Hurted Flag
                    STA        SE_SPRITE_MOVE_SEQ,X ; Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move
                    BRA        PBAIM_ES_End
*-                    
PBAIM_Death         JSR        GetDeathSequenceID   ; Get Death Sequence ID (from Stand-Up or Duck)
                    STA        SE_SPRITE_MOVE_SEQ,X ; Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move
*-         
PBAIM_ES_End        STZ        SE_SPRITE_NEXT_SEQ,X
                    PLX
                    BRA        PBAIM_End
*--
PBAIM_ApplyInput    LDA        SE_SPRITE_ID,X       ; Sprite ID
                    LDY        SE_SPRITE_X0,X       ; Sprite X0 
                    LDX        SE_Player1Input      ; Input = 000000KP|0x31-0x39
                    JSR        GetPlayerSequence    ; Guess a SequenceID based on Input
                    PLX
                    CMP        #$0000               ; No Sequence ID
                    BEQ        PBAIM_End
                    STA        SE_SPRITE_MOVE_SEQ,X ; New Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move
*--
PBAIM_End           RTS


**********************************************************************************
*  PlayerBlueAIReceive() :  Human Player Blue IA Receive function (IN:X=Sprite). *
**********************************************************************************
PlayerBlueAIReceive STX        PBAIR_SPRITE_INDEX   ; React to Collision Received from Red Players
*--
PBAIR_Check_Hit     STZ        PBAIR_RED1_SCORE     ; Check if the Blue Player has been hit by one Red Player
                    STZ        PBAIR_RED2_SCORE
                    STZ        PBAIR_SEND_INDEX     ; Sender Sprite Index*2
                    LDA        SE_MAX_SPRITE_NB
                    ASL
                    STA        PBAIR_Check_Hit_1+1
                    ASL
                    CLC
PBAIR_Check_Hit_1   ADC        #$AAAA               ; Table Size (NbSprite*6)
                    ADC        SE_SPRITE_RECV,X     ; Table Start @
                    STA        PBAIR_Next_Limit+1   ; Table End @
                    LDA        SE_SPRITE_RECV,X     
                    TAY
PBAIR_Loop          LDA:       $0000,Y              ; Score
                    BEQ        PBAIR_Continue
                    LDA        PBAIR_SEND_INDEX
                    CMP        #$0010               ; Hit by Player Red #1
                    BEQ        PBAIR_Red1
                    CMP        #$0012               ; Hit by Player Red #2
                    BEQ        PBAIR_Red2
                    BRA        PBAIR_Continue
*-
PBAIR_Red1          LDA:       $0000,Y              ; Score Red 1
                    STA        PBAIR_RED1_SCORE
                    LDA:       $0002,Y              ; Max Score Red 1
                    STA        PBAIR_RED1_SCOREMAX
                    LDA:       $0004,Y              ; Impact Screen @ Red 1
                    STA        PBAIR_RED1_IMPACT
                    BRA        PBAIR_Continue
*-
PBAIR_Red2          LDA:       $0000,Y              ; Score Red 2
                    STA        PBAIR_RED2_SCORE
                    LDA:       $0002,Y              ; Max Score Red 2
                    STA        PBAIR_RED2_SCOREMAX
                    LDA:       $0004,Y              ; Impact Screen @ Red 2
                    STA        PBAIR_RED2_IMPACT
*-
PBAIR_Continue      INC        PBAIR_SEND_INDEX     ; Next Send Sprite
                    INC        PBAIR_SEND_INDEX
                    TYA
                    CLC
                    ADC        #$0006               ; Entry Size
                    TAY
PBAIR_Next_Limit    CPY        #$AAAA               ; End of Table
                    BNE        PBAIR_Loop
*------
PBAIR_ProcessRed1   LDA        PBAIR_RED1_SCORE     ; Hit by Player Red #1
                    BEQ        PBAIR_ProcessRed2
*--                          
                    LDX        #$000E               ; Blue Player Index
                    JSR        DecrementLifeBar     ; Decrease Blue Player Life Bar
                    LDA        #$0009               ; Add Touched Sound
                    JSR        AppendToSoundList    
*-                  
                    LDA        PBAIR_RED1_SCORE     ; Full Impact ?
                    CMP        PBAIR_RED1_SCOREMAX
                    BNE        PBAIR_ImpactRed1                  
*--  
PBAIR_HurtedRed1    LDX        PBAIR_SPRITE_INDEX   ; Check for Hurted Sequence
                    LDY        #$0010               ; Red #1 Index
                    JSR        GetTouchedSequence
                    BEQ        PBAIR_ImpactRed1                    
*-
                    ORA        #$8000               ; Hurted Flag
                    LDX        PBAIR_SPRITE_INDEX
                    STA        SE_SPRITE_NEXT_SEQ,X ; New pending Hurted Sequence
                    BRA        PBAIR_ProcessRed2
*-                                      
PBAIR_ImpactRed1    LDA        PBAIR_RED1_IMPACT    ; Impact X0|Y0 Address on Screen
                    JSR        AddImpactSprite      ; Create Impact Sprite                                                       
*------
PBAIR_ProcessRed2   LDA        PBAIR_RED2_SCORE     ; Hit by Player Red #2
                    BEQ        PBAIR_End
*--                         
                    LDX        #$000E               ; Blue Player Index
                    JSR        DecrementLifeBar     ; Decrease Blue Player Life Bar
                    LDA        #$0009               ; Add Touched Sound
                    JSR        AppendToSoundList
*-                  
                    LDA        PBAIR_RED2_SCORE     ; Full Impact ?
                    CMP        PBAIR_RED2_SCOREMAX
                    BNE        PBAIR_ImpactRed2
*-
PBAIR_HurtedRed2    LDX        PBAIR_SPRITE_INDEX   ; Check for Hurted Sequence
                    LDA        SE_SPRITE_NEXT_SEQ,X ; Already hurted by Red 1 ?
                    BNE        PBAIR_ImpactRed2     ; Yes => Impact Only
                    LDY        #$0012               ; Red #2 Index
                    JSR        GetTouchedSequence
                    BEQ        PBAIR_ImpactRed2
*-
                    ORA        #$8000               ; Hurted Flag
                    LDX        PBAIR_SPRITE_INDEX
                    STA        SE_SPRITE_NEXT_SEQ,X ; New pending Hurted Sequence
                    BRA        PBAIR_End
*-
PBAIR_ImpactRed2    LDA        PBAIR_RED2_IMPACT    ; Impact X0|Y0 Address on Screen
                    JSR        AddImpactSprite      ; Create Impact Sprite
*----
PBAIR_End           LDX        PBAIR_SPRITE_INDEX
                    RTS

*----------   PlayerBlueAIReceive Variables   ---------
PBAIR_SPRITE_INDEX  HEX        0000                 ; Current Sprite Index (0,2,4,6...)
PBAIR_SEND_INDEX    HEX        0000                 ; Send Sprite Index (0,2,4,6...)
PBAIR_RED1_SCORE    HEX        0000                 ; Score
PBAIR_RED1_SCOREMAX HEX        0000                 ; Max Score
PBAIR_RED1_IMPACT   HEX        0000                 ; Impact @
PBAIR_RED2_SCORE    HEX        0000                 ; Score
PBAIR_RED2_SCOREMAX HEX        0000                 ; Max Score
PBAIR_RED2_IMPACT   HEX        0000                 ; Impact @


**************************************************************************
*  PlayerRedAIMove() :  Human Player Red IA Move function (IN:X=Sprite). *
**************************************************************************
PlayerRedAIMove     PHX                             ; User Player Red Moves
*--
                    JSR        BackupSpriteInfo     ; Backup Sprite info
*--
PRAIM_ExtraSequence LDA        SE_SPRITE_NEXT_SEQ,X ; Any Sequence pending ?
                    BEQ        PRAIM_ApplyInput
                    CMP        #$FFFF               ; Red User player has died
                    BEQ        PRAIM_Death
                    AND        #$8000               ; Red User player has been hurted             
                    BNE        PRAIM_Hurted                    
                    BRA        PRAIM_ES_End
*----
PRAIM_Hurted        LDA        SE_SPRITE_NEXT_SEQ,X ; Hurted Sequence ID (from Stand-Up)
                    AND        #$7FFF               ; Clear Hurted Flag
                    STA        SE_SPRITE_MOVE_SEQ,X ; Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move
                    BRA        PRAIM_ES_End
*-                    
PRAIM_Death         JSR        GetDeathSequenceID   ; Get Death Sequence ID (from Stand-Up or Duck)
                    STA        SE_SPRITE_MOVE_SEQ,X ; Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move
*-         
PRAIM_ES_End        STZ        SE_SPRITE_NEXT_SEQ,X
                    PLX
                    BRA        PRAIM_End
*----
PRAIM_ApplyInput    LDA        SE_SPRITE_ID,X       ; Sprite ID
                    LDY        SE_SPRITE_X0,X       ; Sprite X0 
                    LDX        SE_Player2Input      ; Input = 000000KP|0x31-0x39
                    JSR        GetPlayerSequence    ; Guess a SequenceID based on Input
                    PLX
                    CMP        #$0000               ; No Sequence ID
                    BEQ        PRAIM_End
                    STA        SE_SPRITE_MOVE_SEQ,X ; New Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move            
*----
PRAIM_End           RTS


********************************************************************************
*  PlayerRedAIReceive() :  Human Player Red IA Receive function (IN:X=Sprite). *
********************************************************************************
PlayerRedAIReceive  STX        PRAIR_SPRITE_INDEX   ; React to Collision Received from Blue Player
*--
PRAIR_Check_Hit     STZ        PRAIR_BLUE_SCORE     ; Check if the Red Player has been hit by the Blue Player
                    STZ        PRAIR_SEND_INDEX     ; Sender Sprite Index*2
                    LDA        SE_MAX_SPRITE_NB
                    ASL
                    STA        PRAIR_Check_Hit_1+1
                    ASL
                    CLC
PRAIR_Check_Hit_1   ADC        #$AAAA               ; Table Size (NbSprite*6)
                    ADC        SE_SPRITE_RECV,X     ; Table Start @
                    STA        PRAIR_Next_Limit+1   ; Table End @
                    LDA        SE_SPRITE_RECV,X     
                    TAY
PRAIR_Loop          LDA:       $0000,Y              ; Score
                    BEQ        PRAIR_Continue
                    LDA        PRAIR_SEND_INDEX
                    CMP        #$000E               ; Hit by Player Blue
                    BEQ        PRAIR_ProcessBlue
PRAIR_Continue      INC        PRAIR_SEND_INDEX     ; Next Send Sprite
                    INC        PRAIR_SEND_INDEX
                    TYA
                    CLC
                    ADC        #$0006               ; Entry Size
                    TAY
PRAIR_Next_Limit    CPY        #$AAAA               ; End of Table
                    BNE        PRAIR_Loop
                    BRA        PRAIR_End
*----
PRAIR_ProcessBlue   LDA:       $0000,Y              ; Score
                    STA        PRAIR_BLUE_SCORE
                    LDA:       $0002,Y              ; Max Score
                    STA        PRAIR_BLUE_SCOREMAX
                    LDA:       $0004,Y              ; Impact Screen @
                    STA        PRAIR_BLUE_IMPACT
*--
                    LDA        PRAIR_BLUE_SCORE     ; Decrease Red Player Life Bar
                    LDX        #$0010               ; Red Player #1 Index
                    JSR        DecrementLifeBar     
                    LDA        #$0009               ; Add Touched Sound
                    JSR        AppendToSoundList                                 
                    LDA        PRAIR_BLUE_SCORE     ; Full Impact ?
                    CMP        PRAIR_BLUE_SCOREMAX
                    BNE        PRAIR_Impact
*--  
                    LDX        PRAIR_SPRITE_INDEX   ; Check for Hurted Sequence
                    LDY        #$000E               ; Blue Index
                    JSR        GetTouchedSequence
                    BEQ        PRAIR_Impact
*-
PRAIR_Hurted        ORA        #$8000               ; Hurted Flag
                    LDX        PRAIR_SPRITE_INDEX
                    STA        SE_SPRITE_NEXT_SEQ,X ; New pending Hurted Sequence
                    BRA        PRAIR_End
*-
PRAIR_Impact        LDA        PRAIR_BLUE_IMPACT    ; Impact X0|Y0 Address on Screen
                    JSR        AddImpactSprite      ; Create Impact Sprite
*----
PRAIR_End           LDX        PRAIR_SPRITE_INDEX
                    RTS

*----------   PlayerRedAIReceive Variables   ---------
PRAIR_SPRITE_INDEX  HEX        0000                 ; Current Sprite Index (0,2,4,6...)
PRAIR_SEND_INDEX    HEX        0000                 ; Send Sprite Index (0,2,4,6...)
PRAIR_BLUE_SCORE    HEX        0000                 ; Score
PRAIR_BLUE_SCOREMAX HEX        0000                 ; Max Score 
PRAIR_BLUE_IMPACT   HEX        0000                 ; Impact @


*********************************************************************************
*  OpponentBlueAIMove() :  Computer Player Blue IA Move function (IN:X=Sprite). *
*********************************************************************************
OpponentBlueAIMove  PHX                             ; Computer Player Blue Moves
*--
                    JSR        BackupSpriteInfo     ; Backup Sprite info
*--
OBAIM_ExtraSequence LDA        SE_SPRITE_NEXT_SEQ,X ; Any Sequence pending ?
                    BEQ        OBAIM_GuessMove
                    CMP        #$FFFF               ; Blue Computer player has died
                    BEQ        OBAIM_Death
                    AND        #$8000               ; Blue Computer player has been hurted             
                    BNE        OBAIM_Hurted                    
                    BRA        OBAIM_ES_End
*-
OBAIM_Hurted        LDA        SE_SPRITE_NEXT_SEQ,X ; Hurted Sequence ID (from Stand-Up)
                    STA        SE_SPRITE_MOVE_SEQ,X ; Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move
                    BRA        OBAIM_ES_End
*-                    
OBAIM_Death         JSR        GetDeathSequenceID   ; Get Death Sequence ID (from Stand-Up or Duck)
                    STA        SE_SPRITE_MOVE_SEQ,X ; Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move
*-         
OBAIM_ES_End        STZ        SE_SPRITE_NEXT_SEQ,X
                    PLX
                    BRA        OBAIM_End
*--
OBAIM_GuessMove     LDX        #$000E               ; Current Sprite Index (Blue Computer)
                    LDY        #$0010               ; Opponent Sprite Index (Red 1 Computer)
                    JSR        GetPlayerIAMove      ; Guess next Move (000000KP|0x31-0x39) for this Player
                    STA        OBAIM_ApplyInput_1+1
*--
OBAIM_ApplyInput    PLX                             ; Get back Sprite Index
                    PHX
                    LDA        SE_SPRITE_ID,X       ; Sprite ID
                    LDY        SE_SPRITE_X0,X       ; Sprite X0 
OBAIM_ApplyInput_1  LDX        #$AAAA               ; Input = 000000KP|0x31-0x39
                    JSR        GetPlayerSequence    ; Guess a SequenceID based on Input
                    PLX
                    CMP        #$0000               ; No Sequence ID
                    BEQ        OBAIM_End
                    STA        SE_SPRITE_MOVE_SEQ,X ; New Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move            
*--
OBAIM_End           RTS


***********************************************************************************
*  OpponentRed1AIMove() :  Computer Player Red #1 IA Move function (IN:X=Sprite). *
***********************************************************************************
OpponentRed1AIMove  PHX                             ; Computer Player Red #1 Moves
*--
                    JSR        BackupSpriteInfo     ; Backup Sprite info
*--
OR1AIM_ExtraSeq     LDA        SE_SPRITE_NEXT_SEQ,X ; Any Sequence pending ?
                    BEQ        OR1AIM_GuessMove
                    CMP        #$FFFF               ; Red Computer player has died
                    BEQ        OR1AIM_Death
                    AND        #$8000               ; Red Computer player has been hurted             
                    BNE        OR1AIM_Hurted                    
                    BRA        OR1AIM_ES_End
*-
OR1AIM_Hurted       LDA        SE_SPRITE_NEXT_SEQ,X ; Hurted Sequence ID (from Stand-Up)
                    STA        SE_SPRITE_MOVE_SEQ,X ; Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move
                    BRA        OR1AIM_ES_End
*-                    
OR1AIM_Death        JSR        GetDeathSequenceID   ; Get Death Sequence ID (from Stand-Up or Duck)
                    STA        SE_SPRITE_MOVE_SEQ,X ; Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move
*-         
OR1AIM_ES_End       STZ        SE_SPRITE_NEXT_SEQ,X
                    PLX
                    BRA        OR1AIM_End
*--
OR1AIM_GuessMove    LDX        #$0010               ; Current Sprite Index (Red 1 Computer)
                    LDY        #$000E               ; Opponent Sprite Index (Blue)
                    JSR        GetPlayerIAMove      ; Guess next Move (000000KP|0x31-0x39) for this Player
                    STA        OR1AIM_ApplyInput_1+1
*--
OR1AIM_ApplyInput   PLX                             ; Get back Sprite Index
                    PHX
                    LDA        SE_SPRITE_ID,X       ; Sprite ID
                    LDY        SE_SPRITE_X0,X       ; Sprite X0 
OR1AIM_ApplyInput_1 LDX        #$AAAA               ; Input = 000000KP|0x31-0x39
                    JSR        GetPlayerSequence    ; Guess a SequenceID based on Input
                    PLX
                    CMP        #$0000               ; No Sequence ID
                    BEQ        OR1AIM_End
                    STA        SE_SPRITE_MOVE_SEQ,X ; New Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move           
*--
OR1AIM_End          RTS


**********************************************************************************
*  OpponentRed1AIReceive() :  Opponent Red #1 IA Receive function (IN:X=Sprite). *
**********************************************************************************
OpponentRed1AIReceive STX      OR1AIR_SPRITE_INDEX  ; React to Collision Received from Blue Player
*--
OR1AIR_Check_Hit    STZ        OR1AIR_BLUE_SCORE    ; Check if the Red Opponent #1 has been hit by the Blue Player
                    STZ        OR1AIR_SEND_INDEX    ; Sender Sprite Index*2
                    LDA        SE_MAX_SPRITE_NB
                    ASL
                    STA        OR1AIR_Check_Hit_1+1
                    ASL
                    CLC
OR1AIR_Check_Hit_1  ADC        #$AAAA               ; Table Size (NbSprite*6)
                    ADC        SE_SPRITE_RECV,X     ; Table Start @
                    STA        OR1AIR_Next_Limit+1  ; Table End @
                    LDA        SE_SPRITE_RECV,X     
                    TAY
OR1AIR_Loop         LDA:       $0000,Y              ; Score
                    BEQ        OR1AIR_Continue
                    LDA        OR1AIR_SEND_INDEX
                    CMP        #$000E               ; Hit by Player Blue
                    BEQ        OR1AIR_ProcessBlue
OR1AIR_Continue     INC        OR1AIR_SEND_INDEX    ; Next Send Sprite
                    INC        OR1AIR_SEND_INDEX
                    TYA
                    CLC
                    ADC        #$0006               ; Entry Size
                    TAY
OR1AIR_Next_Limit   CPY        #$AAAA               ; End of Table
                    BNE        OR1AIR_Loop
                    BRA        OR1AIR_End
*----                                             
OR1AIR_ProcessBlue  LDA:       $0000,Y              ; Score
                    STA        OR1AIR_BLUE_SCORE
                    LDA:       $0002,Y              ; Max Score
                    STA        OR1AIR_BLUE_SCOREMAX
                    LDA:       $0004,Y              ; Impact Screen @
                    STA        OR1AIR_BLUE_IMPACT
*--
                    LDA        OR1AIR_BLUE_SCORE    ; Decrease Opponent Red #1 Life Bar
                    LDX        #$0010               ; Opponent Red #1 Index                    
                    JSR        DecrementLifeBar     
                    LDA        #$0009               ; Add Touched Sound
                    JSR        AppendToSoundList  
*--
                    LDA        OR1AIR_BLUE_SCORE    ; Full Impact ?
                    CMP        OR1AIR_BLUE_SCOREMAX
                    BNE        OR1AIR_Medium
*--  
OR1AIR_Full         LDA        #$0002               ; Full Impact => Increment Score +200
                    JSR        IncrementScore
                    LDX        OR1AIR_SPRITE_INDEX  ; Check for Hurted Sequence
                    LDY        #$000E               ; Blue Index
                    JSR        GetTouchedSequence
                    BEQ        OR1AIR_Impact
*-
OR1AIR_Hurted       ORA        #$8000               ; Hurted Flag
                    LDX        OR1AIR_SPRITE_INDEX
                    STA        SE_SPRITE_NEXT_SEQ,X ; New pending Hurted Sequence
                    BRA        OR1AIR_End
*-
OR1AIR_Medium       LDA        #$0001               ; Medium Impact => Increment Score +100
                    JSR        IncrementScore
*-
OR1AIR_Impact       LDA        OR1AIR_BLUE_IMPACT   ; Impact X0|Y0 Address on Screen
                    JSR        AddImpactSprite      ; Create Impact Sprite
*----
OR1AIR_End          LDX        OR1AIR_SPRITE_INDEX
                    RTS

*----------   OpponentRed1AIReceive Variables   ---------
OR1AIR_SPRITE_INDEX HEX        0000                 ; Current Sprite Index (0,2,4,6...)
OR1AIR_SEND_INDEX   HEX        0000                 ; Send Sprite Index (0,2,4,6...)
OR1AIR_BLUE_SCORE   HEX        0000                 ; Score
OR1AIR_BLUE_SCOREMAX HEX       0000                 ; Max Score 
OR1AIR_BLUE_IMPACT  HEX        0000                 ; Impact @


***********************************************************************************
*  OpponentRed2AIMove() :  Computer Player Red #2 IA Move function (IN:X=Sprite). *
***********************************************************************************
OpponentRed2AIMove  PHX                             ; Computer Player Red #2 Moves
*--
                    JSR        BackupSpriteInfo     ; Backup Sprite info
*--
OR2AIM_ExtraSeq     LDA        SE_SPRITE_NEXT_SEQ,X ; Any Sequence pending ?
                    BEQ        OR2AIM_GuessMove
                    CMP        #$FFFF               ; Red Computer player #2 has died
                    BEQ        OR2AIM_Death
                    AND        #$8000               ; Red Computer player #2 has been hurted             
                    BNE        OR2AIM_Hurted                    
                    BRA        OR2AIM_ES_End
*-
OR2AIM_Hurted       LDA        SE_SPRITE_NEXT_SEQ,X ; Hurted Sequence ID (from Stand-Up)
                    STA        SE_SPRITE_MOVE_SEQ,X ; Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move
                    BRA        OR2AIM_ES_End
*-                    
OR2AIM_Death        JSR        GetDeathSequenceID   ; Get Death Sequence ID (from Stand-Up or Duck)
                    STA        SE_SPRITE_MOVE_SEQ,X ; Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move
*-         
OR2AIM_ES_End       STZ        SE_SPRITE_NEXT_SEQ,X
                    PLX
                    BRA        OR2AIM_End
*--
OR2AIM_GuessMove    LDX        #$0012               ; Current Sprite Index (Red 2 Computer)
                    LDY        #$000E               ; Opponent Sprite Index (Blue)
                    JSR        GetPlayerIAMove      ; Guess next Move (000000KP|0x31-0x39) for this Player
                    STA        OR2AIM_ApplyInput_1+1
*--
OR2AIM_ApplyInput   PLX                             ; Get back Sprite Index
                    PHX
                    LDA        SE_SPRITE_ID,X       ; Sprite ID
                    LDY        SE_SPRITE_X0,X       ; Sprite X0 
OR2AIM_ApplyInput_1 LDX        #$AAAA               ; Input = 000000KP|0x31-0x39
                    JSR        GetPlayerSequence    ; Guess a SequenceID based on Input
                    PLX
                    CMP        #$0000               ; No Sequence ID
                    BEQ        OR2AIM_End
                    STA        SE_SPRITE_MOVE_SEQ,X ; New Sequence ID (1-N)
                    LDA        #$0001               ; Frame # (1-N)
                    STA        SE_SPRITE_MOVE_FRAM,X
                    LDA        #$0003
                    STA        SE_SPRITE_STATUS,X   ; Move           
*--
OR2AIM_End          RTS


**********************************************************************************
*  OpponentRed2AIReceive() :  Opponent Red #2 IA Receive function (IN:X=Sprite). *
**********************************************************************************
OpponentRed2AIReceive STX      OR2AIR_SPRITE_INDEX  ; React to Collision Received from Blue Player
*--
OR2AIR_Check_Hit    STZ        OR2AIR_BLUE_SCORE    ; Check if the Red Opponent #2 has been hit by the Blue Player
                    STZ        OR2AIR_SEND_INDEX    ; Sender Sprite Index*2
                    LDA        SE_MAX_SPRITE_NB
                    ASL
                    STA        OR2AIR_Check_Hit_1+1
                    ASL
                    CLC
OR2AIR_Check_Hit_1  ADC        #$AAAA               ; Table Size (NbSprite*6)
                    ADC        SE_SPRITE_RECV,X     ; Table Start @
                    STA        OR2AIR_Next_Limit+1  ; Table End @
                    LDA        SE_SPRITE_RECV,X     
                    TAY
OR2AIR_Loop         LDA:       $0000,Y              ; Score
                    BEQ        OR2AIR_Continue
                    LDA        OR2AIR_SEND_INDEX
                    CMP        #$000E               ; Hit by Player Blue
                    BEQ        OR2AIR_ProcessBlue
OR2AIR_Continue     INC        OR2AIR_SEND_INDEX    ; Next Send Sprite
                    INC        OR2AIR_SEND_INDEX
                    TYA
                    CLC
                    ADC        #$0006               ; Entry Size
                    TAY
OR2AIR_Next_Limit   CPY        #$AAAA               ; End of Table
                    BNE        OR2AIR_Loop
                    BRA        OR2AIR_End
*----                                             
OR2AIR_ProcessBlue  LDA:       $0000,Y              ; Score
                    STA        OR2AIR_BLUE_SCORE
                    LDA:       $0002,Y              ; Max Score
                    STA        OR2AIR_BLUE_SCOREMAX
                    LDA:       $0004,Y              ; Impact Screen @
                    STA        OR2AIR_BLUE_IMPACT
*--
                    LDA        OR2AIR_BLUE_SCORE    ; Decrease Opponent Red #1 Life Bar
                    LDX        #$0012               ; Opponent Red #2 Index                    
                    JSR        DecrementLifeBar     
                    LDA        #$0009               ; Add Touched Sound
                    JSR        AppendToSoundList  
*--
                    LDA        OR2AIR_BLUE_SCORE    ; Full Impact ?
                    CMP        OR2AIR_BLUE_SCOREMAX
                    BNE        OR2AIR_Medium
*--  
OR2AIR_Full         LDA        #$0002               ; Full Impact => Increment Score +200
                    JSR        IncrementScore
                    LDX        OR2AIR_SPRITE_INDEX  ; Check for Hurted Sequence
                    LDY        #$000E               ; Blue Index
                    JSR        GetTouchedSequence
                    BEQ        OR2AIR_Impact
*-
OR2AIR_Hurted       ORA        #$8000               ; Hurted Flag
                    LDX        OR2AIR_SPRITE_INDEX
                    STA        SE_SPRITE_NEXT_SEQ,X ; New pending Hurted Sequence
                    BRA        OR2AIR_End
*-
OR2AIR_Medium       LDA        #$0001               ; Medium Impact => Increment Score +100
                    JSR        IncrementScore
*-
OR2AIR_Impact       LDA        OR2AIR_BLUE_IMPACT   ; Impact X0|Y0 Address on Screen
                    JSR        AddImpactSprite      ; Create Impact Sprite
*----
OR2AIR_End          LDX        OR2AIR_SPRITE_INDEX
                    RTS

*----------   OpponentRed2AIReceive Variables   ---------
OR2AIR_SPRITE_INDEX HEX        0000                 ; Current Sprite Index (0,2,4,6...)
OR2AIR_SEND_INDEX   HEX        0000                 ; Send Sprite Index (0,2,4,6...)
OR2AIR_BLUE_SCORE   HEX        0000                 ; Score
OR2AIR_BLUE_SCOREMAX HEX       0000                 ; Max Score 
OR2AIR_BLUE_IMPACT  HEX        0000                 ; Impact @


*******************************************************************************************************
*  GetPlayerIAMove() :  Compute IA Player next Move (IN:X=PlayerIndex,IN:Y=OpponentIndex,OUT:A=Move). *
*******************************************************************************************************
GetPlayerIAMove     STX        GPIAM_PlayerIndex    ; Init
                    STY        GPIAM_OpponentIndex
*----
GPIAM_Wait          LDA        SE_SPRITE_MOVE_WAIT,X ; Wait between each Move
                    BEQ        GPIAM_WaitInit
                    DEC
                    STA        SE_SPRITE_MOVE_WAIT,X
                    JMP        GPIAM_Meditation
GPIAM_WaitInit      LDA        PG_Level             ; Reset Counter with appropriate value
                    JSR        GetLevelMoveWait
                    STA        SE_SPRITE_MOVE_WAIT,X
*----
GPIAM_Pending       LDA        SE_SPRITE_MOVE,X     ; Do we have a pending Move in the list ?
                    STA        GPIAM_Pending_1+1    ; Move Index @
                    STA        GPIAM_Pending_3+1
                    STA        GPIAM_Pending_4+1
                    STA        GPIAM_Pending_End+1
                    INC
                    INC
                    STA        GPIAM_Pending_2+1    ; Move List @
GPIAM_Pending_1     LDA        $AAAA                ; Next Move Index 
                    BEQ        GPIAM_GuessMove      ; No Pending Move
                    DEC
                    DEC
                    TAY
GPIAM_Pending_2     LDA        $AAAA,Y              ; Next Move
                    BEQ        GPIAM_Pending_End    ; No more move
                    STA        GPIAM_End+1
GPIAM_Pending_3     INC        $AAAA                ; Prepare Next Move index
GPIAM_Pending_4     INC        $AAAA
                    JMP        GPIAM_End
GPIAM_Pending_End   STZ        $AAAA                ; Reset Next Move List                 
*----
GPIAM_GuessMove     LDX        GPIAM_PlayerIndex    ; Get Opponent location
                    LDY        GPIAM_OpponentIndex
                    JSR        GetOpponentLocation
                    STA        GPIAM_OpponentLocat
                    LDX        GPIAM_PlayerIndex    ; Check Space Behind Player
                    JSR        HasSpaceBehind
                    STA        GPIAM_HSB_Player
                    LDX        GPIAM_HSB_Opponent   ; Check Space Behind Opponent
                    JSR        HasSpaceBehind
                    STA        GPIAM_HSB_Opponent
                    LDX        GPIAM_PlayerIndex    ; Is Player in Duck Position ?
                    LDA        SE_SPRITE_ID,X
                    JSR        IsDuckSprite
                    ROL
                    AND        #$0001
                    STA        GPIAM_Duck_Player
*-
                    LDA        GPIAM_HSB_Player     ; Combine 0004 and 0006 position + Space Behind Opponent
                    BEQ        GPIAM_Direction
                    LDA        GPIAM_OpponentLocat
                    CMP        #$0004
                    BEQ        GPIAM_SpaceBehindOp
                    CMP        #$0006
                    BEQ        GPIAM_SpaceBehindOp                    
                    BRA        GPIAM_Direction
GPIAM_SpaceBehindOp ORA        #$8000               ; Add Space Behind Opponent flag
                    STA        GPIAM_OpponentLocat
*--                    
GPIAM_Direction     STZ        GPIAM_Player_Direct  ; Get Player Direction
                    LDX        GPIAM_PlayerIndex
                    LDA        SE_SPRITE_ID,X
                    JSR        GetSpriteDirection
                    BCC        GPIAM_Location
                    INC        GPIAM_Player_Direct
*----
GPIAM_Location      LDX        #$0000
GPIAM_LocationLoop  LDA        GPIAM_TabLocation,X
                    CMP        GPIAM_OpponentLocat
                    BEQ        GPIAM_Location_1
                    INX
                    INX
                    BRA        GPIAM_LocationLoop
GPIAM_Location_1    LDA        GPIAM_Player_Direct
                    BEQ        GPIAM_Location_L
*-
GPIAM_Location_R    LDA        GPIAM_Duck_Player    ; Player goes Right
                    BNE        GPIAM_Location_R_D
                    LDA        GPIAM_TabLocRight,X  ; Right + Stand-up                  
                    BRA        GPIAM_Location_3
GPIAM_Location_R_D  LDA        GPIAM_TabLocRightD,X ; Right + Duck
                    BRA        GPIAM_Location_3 
*-
GPIAM_Location_L    LDA        GPIAM_Duck_Player    ; Player goes Left
                    BNE        GPIAM_Location_L_D                 
                    LDA        GPIAM_TabLocLeft,X   ; Left + Stand-up    
                    BRA        GPIAM_Location_3 
GPIAM_Location_L_D  LDA        GPIAM_TabLocLeftD,X  ; Left + Duck
*-
GPIAM_Location_3    TAY
                    INC
                    INC
                    STA        GPIAM_Location_Mult+1
                    LDA:       $0000,Y              ; Number of possible Moves
                    CMP        #$0001
                    BEQ        GPIAM_Location_One
                    LDX        GPIAM_PlayerIndex    ; Get Ramdom Index
                    JSR        GetRandomIndex
                    TAY
GPIAM_Location_Mult LDA        $AAAA,Y
                    BRA        GPIAM_Install_Init                    
GPIAM_Location_One  LDA:       $0002,Y              ; Only possible move = First one
GPIAM_Install_Init  STA        GPIAM_InstallMove+1
                    INC
                    INC            
                    STA        GPIAM_IM_Loop+1
                    LDX        GPIAM_PlayerIndex
                    LDA        SE_SPRITE_MOVE,X
                    STA        GPIAM_IM_Index+1
                    INC
                    INC
                    STA        GPIAM_IM_List+1
*--
GPIAM_InstallMove   LDA        $AAAA                ; Install Moves List
                    ASL                             ; Nb Seq ID * 2
                    TAY
GPIAM_IM_Loop       LDA        $AAAA,Y              ; Sequence ID list
GPIAM_IM_List       STA        $BBBB,Y              ; SE_SPRITE_MOVEx+2
                    DEY
                    DEY
                    BPL        GPIAM_IM_Loop
                    LDA        #$0002               ; Init Move Index 
GPIAM_IM_Index      STA        $BBBB                ; SE_SPRITE_MOVEx
*----
GPIAM_Meditation    LDA        #$0035               ; Do Nothing
                    STA        GPIAM_End+1
*----
GPIAM_End           LDA        #$AAAA               ; Move (000000KP|0x31-0x39)
                    RTS

*----------   GetPlayerIAMove Variables   ---------
GPIAM_PlayerIndex   HEX        0000                 ; IA Player Sprite Index
GPIAM_OpponentIndex HEX        0000                 ; Opponent Player (IA or Human) Sprite Index 
GPIAM_OpponentLocat HEX        0000                 ; Opponent Location 
GPIAM_HSB_Player    HEX        0000                 ; Has Space Behind Player 
GPIAM_HSB_Opponent  HEX        0000                 ; Has Space Behind Opponent 
GPIAM_Player_Direct HEX        0000                 ; Player Direction (0:Left, 1:Right)
GPIAM_Duck_Player   HEX        0000                 ; Player is in Duck position 
GPIAM_TabLocation   HEX        0700,0100,0480,0400,4400,4404,4444,0500,0800
                    HEX        0200,0900,0300,0680,0600,6600,6606,6666
*--
GPIAM_TabLocRight   DA         GPIAM_R_0007,GPIAM_R_0001,GPIAM_R_8004,GPIAM_R_0004,GPIAM_R_0044,GPIAM_R_0444,GPIAM_R_4444,GPIAM_R_0005
                    DA         GPIAM_R_0008,GPIAM_R_0002,GPIAM_R_0009,GPIAM_R_0003,GPIAM_R_8006,GPIAM_R_0006,GPIAM_R_0066,GPIAM_R_0666,GPIAM_R_6666 
*-
GPIAM_R_0009        HEX        0800
                    DA         GPIAM_R_Move_FlK,GPIAM_A_Move_Pro,GPIAM_R_Move_HiP,GPIAM_R_Move_HiP ; Flight Kick Right[*], Protect[*], High Puch Right[**]
                    DA         GPIAM_R_Move_HiK,GPIAM_R_Move_HiK,GPIAM_A_Move_Duc,GPIAM_R_Move_MiK ; High Kick Right[**], Go Duck[*], Middle Kick Right[*]
GPIAM_R_0003        HEX        0800
                    DA         GPIAM_R_Move_StB,GPIAM_R_Move_LoK,GPIAM_R_Move_LoK,GPIAM_R_Move_MiP ; Step Back[*], Low Kick Right[**], Middle Puch Right[*]
                    DA         GPIAM_R_Move_Som,GPIAM_R_Move_SoB,GPIAM_R_Move_DuK,GPIAM_R_Move_DuP ; Sommersault Right[*], Sommersault Back[*], Duck + Kick Right[*], Duck + Punch Right[*]
GPIAM_R_8006        HEX        1000                 ; Has Space behind opponent
                    DA         GPIAM_R_Move_HiP,GPIAM_R_Move_MiP,GPIAM_R_Move_HiK,GPIAM_R_Move_MiK ; High Puch Right[*], Middle Punch Right[*], High Kick Right[*], Middle Kick Right[*]
                    DA         GPIAM_R_Move_LoK,GPIAM_R_Move_FlK,GPIAM_A_Move_Pro,GPIAM_R_Move_StB ; Low Kick Right[*], Flight Kick Right[*], Protect[*], Step Back[*]
                    DA         GPIAM_R_Move_CxA,GPIAM_R_Move_CxB,GPIAM_R_Move_CxC,GPIAM_R_Move_CxD ; ComplexA Left[*], ComplexB Left[*], ComplexC Left[*], ComplexD Left[*]
                    DA         GPIAM_R_Move_CxE,GPIAM_R_Move_CxF,GPIAM_R_Move_CxG,GPIAM_R_Move_CxH ; ComplexE Left[*], ComplexF Left[*], ComplexG Left[*], ComplexH Left[*]
GPIAM_R_0006        HEX        1000                 ; No Space behind opponent
                    DA         GPIAM_R_Move_HiP,GPIAM_R_Move_MiP,GPIAM_R_Move_HiK,GPIAM_R_Move_MiK ; High Puch Right[*], Middle Punch Right[*], High Kick Right[*], Middle Kick Right[*]
                    DA         GPIAM_R_Move_LoK,GPIAM_R_Move_FlK,GPIAM_A_Move_Pro,GPIAM_R_Move_StB ; Low Kick Right[*], Flight Kick Right[*], Protect[*], Step Back[*]
                    DA         GPIAM_R_Move_CxA,GPIAM_R_Move_CxB,GPIAM_R_Move_CxC,GPIAM_R_Move_CxD ; ComplexA Left[*], ComplexB Left[*], ComplexC Left[*], ComplexD Left[*]
                    DA         GPIAM_R_Move_CxE,GPIAM_R_Move_CxF,GPIAM_R_Move_CxG,GPIAM_R_Move_CxH ; ComplexE Left[*], ComplexF Left[*], ComplexG Left[*], ComplexH Left[*]
GPIAM_R_0066        HEX        0400
                    DA         GPIAM_R_Move_Wal,GPIAM_R_Move_Som,GPIAM_R_Move_FlK,GPIAM_A_Move_Not ; Walk Right[*], Sommersault Right[*], Flight Kick Right[*], Nothing[*]
GPIAM_R_0666        HEX        0800
                    DA         GPIAM_R_Move_Wal,GPIAM_R_Move_Wal,GPIAM_R_Move_FlK,GPIAM_R_Move_FlK ; Walk Right[**], Flight Kick Right[**], 
                    DA         GPIAM_R_Move_Som,GPIAM_R_Move_Som,GPIAM_A_Move_Not,GPIAM_A_Move_Not ; Sommersault Right[**], Nothing[**]
GPIAM_R_6666        HEX        0800
                    DA         GPIAM_R_Move_Wal,GPIAM_R_Move_Wal,GPIAM_R_Move_Wal,GPIAM_R_Move_FlK ; Walk Right[***], Flight Kick Right[*]
                    DA         GPIAM_R_Move_Som,GPIAM_R_Move_Som,GPIAM_R_Move_Som,GPIAM_A_Move_Not ; Sommersault Right[***], Nothing[*]
GPIAM_R_0005        HEX        1000
                    DA         GPIAM_R_Move_Wal,GPIAM_R_Move_StB,GPIAM_A_Move_Pro,GPIAM_A_Move_Duc ; Walk Right[*], Step Back[*], Protect[*], Go Duck[*]
                    DA         GPIAM_R_Move_Som,GPIAM_R_Move_SoB,GPIAM_R_Move_BaK,GPIAM_R_Move_BaK ; Sommersault Right[*], Sommersault Back[*], Back Kick Right[*], Middle Puch Right[*]
                    DA         GPIAM_R_Move_Cx1,GPIAM_R_Move_Cx2,GPIAM_R_Move_Cx3,GPIAM_R_Move_Cx4 ; Complex1 Right[*], Complex2 Right[*], Complex3 Right[*], Complex4 Right[*]
                    DA         GPIAM_R_Move_Cx5,GPIAM_R_Move_Cx6,GPIAM_R_Move_Cx7,GPIAM_R_Move_Cx8 ; Complex5 Right[*], Complex6 Right[*], Complex7 Right[*], Complex8 Right[*]
GPIAM_R_0008        HEX        0800
                    DA         GPIAM_R_Move_Wal,GPIAM_R_Move_Wal,GPIAM_R_Move_StB,GPIAM_R_Move_StB ; Walk Right[**], Step Back[**]
                    DA         GPIAM_A_Move_Pro,GPIAM_A_Move_Duc,GPIAM_R_Move_Som,GPIAM_R_Move_SoB ; Protect[*], Go Duck[*], Sommersault Right[*], Sommersault Back[*]
GPIAM_R_0002        HEX        0800
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_R_Move_StB,GPIAM_R_Move_StB ; Walk Right[**], Step Back[**]
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Duc,GPIAM_R_Move_Som,GPIAM_R_Move_SoB ; Jump[*], Go Duck[*], Sommersault Right[*], Sommersault Back[*]
GPIAM_R_0007        HEX        0800
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Duc,GPIAM_R_Move_BaK ; Change Direction[**], Go Duck[*], Back Kick Right[*]
                    DA         GPIAM_R_Move_SoB,GPIAM_R_Move_SoB,GPIAM_A_Move_Pro,GPIAM_A_Move_Not ; Sommersault Back[**], Protect[*], Nothing[*]
GPIAM_R_0001        HEX        0800
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Duc ; Change Direction[***], Go Duck[**]
                    DA         GPIAM_A_Move_Duc,GPIAM_R_Move_SoB,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Sommersault Back[*], Jump[*], Nothing[*]
GPIAM_R_8004        HEX        0800                 ; Has Space behind opponent
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_R_Move_BaK ; Change Direction[***], Back Kick Right[**]
                    DA         GPIAM_R_Move_BaK,GPIAM_A_Move_Duc,GPIAM_R_Move_SoB,GPIAM_A_Move_Not ; Go Duck[*], Sommersault Back[*], Nothing[*]
GPIAM_R_0004        HEX        0800                 ; No Space behind opponent
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_R_Move_BaK ; Change Direction[***], Back Kick Right[**]
                    DA         GPIAM_R_Move_BaK,GPIAM_A_Move_Duc,GPIAM_R_Move_SoB,GPIAM_A_Move_Not ; Go Duck[*], Sommersault Back[*], Nothing[*]
GPIAM_R_0044        HEX        0200
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Not                                   ; Change Direction[*], Nothing[*]
GPIAM_R_0444        HEX        0400
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Not ; Change Direction[***], Nothing[*]
GPIAM_R_4444        HEX        0400
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_R_Move_SoB,GPIAM_A_Move_Not ; Change Direction[**], Sommersault Back[*], Nothing[*]
*--
GPIAM_TabLocLeft    DA         GPIAM_L_0007,GPIAM_L_0001,GPIAM_L_8004,GPIAM_L_0004,GPIAM_L_0044,GPIAM_L_0444,GPIAM_L_4444,GPIAM_L_0005
                    DA         GPIAM_L_0008,GPIAM_L_0002,GPIAM_L_0009,GPIAM_L_0003,GPIAM_L_8006,GPIAM_L_0006,GPIAM_L_0066,GPIAM_L_0666,GPIAM_L_6666 
*-
GPIAM_L_0007        HEX        0800
                    DA         GPIAM_L_Move_FlK,GPIAM_A_Move_Pro,GPIAM_L_Move_HiP,GPIAM_L_Move_HiP ; Flight Kick Left[*], Protect[*], High Puch Left[**]
                    DA         GPIAM_L_Move_HiK,GPIAM_L_Move_HiK,GPIAM_A_Move_Duc,GPIAM_L_Move_MiK ; High Kick Left[**], Go Duck[*], Middle Kick Left[*]
GPIAM_L_0001        HEX        0800
                    DA         GPIAM_L_Move_StB,GPIAM_L_Move_LoK,GPIAM_L_Move_LoK,GPIAM_L_Move_MiP ; Step Back[*], Low Kick Left[**], Middle Puch Left[*]
                    DA         GPIAM_L_Move_Som,GPIAM_L_Move_SoB,GPIAM_L_Move_DuK,GPIAM_L_Move_DuP ; Sommersault Left[*], Sommersault Back[*], Duck + Kick Left[*], Duck + Punch Left[*]
GPIAM_L_8004        HEX        1000                 ; Has space behind opponent
                    DA         GPIAM_L_Move_HiP,GPIAM_L_Move_MiP,GPIAM_L_Move_HiK,GPIAM_L_Move_MiK ; High Puch Left[*], Middle Punch Left[*], High Kick Left[*], Middle Kick Left[*]
                    DA         GPIAM_L_Move_LoK,GPIAM_L_Move_FlK,GPIAM_A_Move_Pro,GPIAM_L_Move_StB ; Low Kick Left[*], Flight Kick Left[*], Protect[*], Step Back[*]
                    DA         GPIAM_L_Move_CxA,GPIAM_L_Move_CxB,GPIAM_L_Move_CxC,GPIAM_L_Move_CxD ; ComplexA Left[*], ComplexB Left[*], ComplexC Left[*], ComplexD Left[*]
                    DA         GPIAM_L_Move_CxE,GPIAM_L_Move_CxF,GPIAM_L_Move_CxG,GPIAM_L_Move_CxH ; ComplexE Left[*], ComplexF Left[*], ComplexG Left[*], ComplexH Left[*]
GPIAM_L_0004        HEX        1000                 ; No space behind opponent
                    DA         GPIAM_L_Move_HiP,GPIAM_L_Move_MiP,GPIAM_L_Move_HiK,GPIAM_L_Move_MiK ; High Puch Left[*], Middle Punch Left[*], High Kick Left[*], Middle Kick Left[*]
                    DA         GPIAM_L_Move_LoK,GPIAM_L_Move_FlK,GPIAM_A_Move_Pro,GPIAM_L_Move_StB ; Low Kick Left[*], Flight Kick Left[*], Protect[*], Step Back[*]
                    DA         GPIAM_L_Move_CxA,GPIAM_L_Move_CxB,GPIAM_L_Move_CxC,GPIAM_L_Move_CxD ; ComplexA Left[*], ComplexB Left[*], ComplexC Left[*], ComplexD Left[*]
                    DA         GPIAM_L_Move_CxE,GPIAM_L_Move_CxF,GPIAM_L_Move_CxG,GPIAM_L_Move_CxH ; ComplexE Left[*], ComplexF Left[*], ComplexG Left[*], ComplexH Left[*]
GPIAM_L_0044        HEX        0400
                    DA         GPIAM_L_Move_Wal,GPIAM_L_Move_Som,GPIAM_L_Move_FlK,GPIAM_A_Move_Not ; Walk Left[*], Sommersault Left[*], Flight Kick Left[*], Nothing[*]
GPIAM_L_0444        HEX        0800
                    DA         GPIAM_L_Move_Wal,GPIAM_L_Move_Wal,GPIAM_L_Move_FlK,GPIAM_L_Move_FlK ; Walk Left[**], Flight Kick Left[**], 
                    DA         GPIAM_L_Move_Som,GPIAM_L_Move_Som,GPIAM_A_Move_Not,GPIAM_A_Move_Not ; Sommersault Left[**], Nothing[**]
GPIAM_L_4444        HEX        0800
                    DA         GPIAM_L_Move_Wal,GPIAM_L_Move_Wal,GPIAM_L_Move_Wal,GPIAM_L_Move_FlK ; Walk Left[***], Flight Kick Left[*]
                    DA         GPIAM_L_Move_Som,GPIAM_L_Move_Som,GPIAM_L_Move_Som,GPIAM_A_Move_Not ; Sommersault Left[***], Nothing[*]
GPIAM_L_0005        HEX        1000
                    DA         GPIAM_L_Move_Wal,GPIAM_L_Move_StB,GPIAM_A_Move_Pro,GPIAM_A_Move_Duc ; Walk Left[*], Step Back[*], Protect[*], Go Duck[*]
                    DA         GPIAM_L_Move_Som,GPIAM_L_Move_SoB,GPIAM_L_Move_BaK,GPIAM_L_Move_BaK ; Sommersault Left[*], Sommersault Back[*], Back Kick Left[*], Middle Puch Left [*]
                    DA         GPIAM_L_Move_Cx1,GPIAM_L_Move_Cx2,GPIAM_L_Move_Cx3,GPIAM_L_Move_Cx4 ; Complex1 Left[*], Complex2 Left[*], Complex3 Left[*], Complex4 Left[*]
                    DA         GPIAM_L_Move_Cx5,GPIAM_L_Move_Cx6,GPIAM_L_Move_Cx7,GPIAM_L_Move_Cx8 ; Complex5 Left[*], Complex6 Left[*], Complex7 Left[*], Complex8 Left[*]
GPIAM_L_0008        HEX        0800
                    DA         GPIAM_L_Move_Wal,GPIAM_L_Move_Wal,GPIAM_L_Move_StB,GPIAM_L_Move_StB ; Walk Left[**], Step Back[**]
                    DA         GPIAM_A_Move_Pro,GPIAM_A_Move_Duc,GPIAM_L_Move_Som,GPIAM_L_Move_SoB ; Protect[*], Go Duck[*], Sommersault Left[*], Sommersault Back[*]
GPIAM_L_0002        HEX        0800
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_L_Move_StB,GPIAM_L_Move_StB ; Walk Left[**], Step Back[**]
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Duc,GPIAM_L_Move_Som,GPIAM_L_Move_SoB ; Jump[*], Go Duck[*], Sommersault Left[*], Sommersault Back[*]
GPIAM_L_0009        HEX        0800
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Duc,GPIAM_L_Move_BaK ; Change Direction[**], Go Duck[*], Back Kick Left[*]
                    DA         GPIAM_L_Move_SoB,GPIAM_L_Move_SoB,GPIAM_A_Move_Pro,GPIAM_A_Move_Not ; Sommersault Back[**], Protect[*], Nothing[*]
GPIAM_L_0003        HEX        0800
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Duc ; Change Direction[***], Go Duck[**]
                    DA         GPIAM_A_Move_Duc,GPIAM_L_Move_SoB,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Sommersault Back[*], Jump[*], Nothing[*]
GPIAM_L_8006        HEX        0800                 ; Has space behind opponent
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_L_Move_BaK ; Change Direction[***], Back Kick Left[**]
                    DA         GPIAM_L_Move_BaK,GPIAM_A_Move_Duc,GPIAM_L_Move_SoB,GPIAM_A_Move_Not ; Go Duck[*], Sommersault Back[*], Nothing[*]
GPIAM_L_0006        HEX        0800                 ; No space behind opponent
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_L_Move_BaK ; Change Direction[***], Back Kick Left[**]
                    DA         GPIAM_L_Move_BaK,GPIAM_A_Move_Duc,GPIAM_L_Move_SoB,GPIAM_A_Move_Not ; Go Duck[*], Sommersault Back[*], Nothing[*]
GPIAM_L_0066        HEX        0200
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Not                                   ; Change Direction[*], Nothing[*]
GPIAM_L_0666        HEX        0400
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_A_Move_Not ; Change Direction[***], Nothing[*]
GPIAM_L_6666        HEX        0400
                    DA         GPIAM_A_Move_Dir,GPIAM_A_Move_Dir,GPIAM_L_Move_SoB,GPIAM_A_Move_Not ; Change Direction[**], Sommersault Back[*], Nothing[*]
*--
GPIAM_TabLocRightD  DA         GPIAM_R_D_0007,GPIAM_R_D_0001,GPIAM_R_D_8004,GPIAM_R_D_0004,GPIAM_R_D_0044,GPIAM_R_D_0444,GPIAM_R_D_4444,GPIAM_R_D_0005
                    DA         GPIAM_R_D_0008,GPIAM_R_D_0002,GPIAM_R_D_0009,GPIAM_R_D_0003,GPIAM_R_D_8006,GPIAM_R_D_0006,GPIAM_R_D_0066,GPIAM_R_D_0666,GPIAM_R_D_6666 
*-
GPIAM_R_D_0009      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_R_Move_MiP,GPIAM_R_Move_MiP,GPIAM_A_Move_Not ; Up[*], Duck Puch Right[**], Nothing[*]
GPIAM_R_D_0003      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_R_Move_MiP,GPIAM_R_Move_MiK,GPIAM_R_Move_MiK ; Up[*], Duck Puch Right[*], Duck Kick Right[**]
GPIAM_R_D_8006      HEX        0400                 ; Has space behind opponent
                    DA         GPIAM_A_Move_Jum,GPIAM_R_Move_MiP,GPIAM_R_Move_MiP,GPIAM_R_Move_MiK ; Up[*], Duck Puch Right[**], Duck Kick Right[*]
GPIAM_R_D_0006      HEX        0400                 ; No space behind opponent
                    DA         GPIAM_A_Move_Jum,GPIAM_R_Move_MiP,GPIAM_R_Move_MiP,GPIAM_R_Move_MiK ; Up[*], Duck Puch Right[**], Duck Kick Right[*]
GPIAM_R_D_0066      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Up[***], Nothing[*]
GPIAM_R_D_0666      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Up[***], Nothing[*]
GPIAM_R_D_6666      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Up[***], Nothing[*]
GPIAM_R_D_0005      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_R_Move_DBK,GPIAM_R_Move_MiP,GPIAM_R_Move_MiK ; Up[*], Duck Back Kick Right[*], Duck Puch Right[*], Duck Kick Right[*]
GPIAM_R_D_0008      HEX        0200
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Not                                   ; Up[*], Nothing[*]
GPIAM_R_D_0002      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_R_Move_DBK,GPIAM_R_Move_MiP,GPIAM_R_Move_MiK ; Up[*], Duck Back Kick Right[*], Duck Puch Right[*], Duck Kick Right[*]
GPIAM_R_D_0007      HEX        0200
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Not                                   ; Up[*], Nothing[*]
GPIAM_R_D_0001      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_R_Move_DBK,GPIAM_R_Move_DBK,GPIAM_A_Move_Not ; Up[*], Duck Back Kick Right[**], Nothing[*]
GPIAM_R_D_8004      HEX        0400                 ; Has space behind opponent
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_R_Move_DBK,GPIAM_A_Move_Not ; Up[**], Duck Back Kick Right[*], Nothing[*]
GPIAM_R_D_0004      HEX        0400                 ; No space behind opponent
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_R_Move_DBK,GPIAM_A_Move_Not ; Up[**], Duck Back Kick Right[*], Nothing[*]
GPIAM_R_D_0044      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Up[***], Nothing[*]
GPIAM_R_D_0444      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Up[***], Nothing[*]
GPIAM_R_D_4444      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Up[***], Nothing[*]
*--
GPIAM_TabLocLeftD   DA         GPIAM_L_D_0007,GPIAM_L_D_0001,GPIAM_L_D_8004,GPIAM_L_D_0004,GPIAM_L_D_0044,GPIAM_L_D_0444,GPIAM_L_D_4444,GPIAM_L_D_0005
                    DA         GPIAM_L_D_0008,GPIAM_L_D_0002,GPIAM_L_D_0009,GPIAM_L_D_0003,GPIAM_L_D_8006,GPIAM_L_D_0006,GPIAM_L_D_0066,GPIAM_L_D_0666,GPIAM_L_D_6666 
*-
GPIAM_L_D_0007      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_L_Move_MiP,GPIAM_L_Move_MiP,GPIAM_A_Move_Not ; Up[*], Duck Puch Left[**], Nothing[*]
GPIAM_L_D_0001      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_L_Move_MiP,GPIAM_L_Move_MiK,GPIAM_L_Move_MiK ; Up[*], Duck Puch Left[*], Duck Kick Left[**]
GPIAM_L_D_8004      HEX        0400                 ; Has space behind opponent
                    DA         GPIAM_A_Move_Jum,GPIAM_L_Move_MiP,GPIAM_L_Move_MiP,GPIAM_L_Move_MiK ; Up[*], Duck Puch Left[**], Duck Kick Left[*]
GPIAM_L_D_0004      HEX        0400                 ; No space behind opponent
                    DA         GPIAM_A_Move_Jum,GPIAM_L_Move_MiP,GPIAM_L_Move_MiP,GPIAM_L_Move_MiK ; Up[*], Duck Puch Left[**], Duck Kick Left[*]
GPIAM_L_D_0044      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Up[***], Nothing[*]
GPIAM_L_D_0444      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Up[***], Nothing[*]
GPIAM_L_D_4444      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Up[***], Nothing[*]
GPIAM_L_D_0005      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_L_Move_DBK,GPIAM_L_Move_MiP,GPIAM_L_Move_MiK ; Up[*], Duck Back Kick Left[*], Duck Puch Left[*], Duck Kick Left[*]
GPIAM_L_D_0008      HEX        0200
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Not                                   ; Up[*], Nothing[*]
GPIAM_L_D_0002      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_L_Move_DBK,GPIAM_L_Move_MiP,GPIAM_L_Move_MiK ; Up[*], Duck Back Kick Left[*], Duck Puch Left[*], Duck Kick Left[*]
GPIAM_L_D_0009      HEX        0200
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Not                                   ; Up[*], Nothing[*]
GPIAM_L_D_0003      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_L_Move_DBK,GPIAM_L_Move_DBK,GPIAM_A_Move_Not ; Up[*], Duck Back Kick Left[**], Nothing[*]
GPIAM_L_D_8006      HEX        0400                 ; Has space behind opponent
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_L_Move_DBK,GPIAM_A_Move_Not ; Up[**], Duck Back Kick Left[*], Nothing[*]
GPIAM_L_D_0006      HEX        0400                 ; No space behind opponent
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_L_Move_DBK,GPIAM_A_Move_Not ; Up[**], Duck Back Kick Left[*], Nothing[*]
GPIAM_L_D_0066      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Up[***], Nothing[*]
GPIAM_L_D_0666      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Up[***], Nothing[*]
GPIAM_L_D_6666      HEX        0400
                    DA         GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Jum,GPIAM_A_Move_Not ; Up[***], Nothing[*]
*----
GPIAM_A_Move_Not    HEX        0100,3500,0000       ; Do Nothing
GPIAM_A_Move_Dir    HEX        0100,3503,0000       ; Change Direction
GPIAM_A_Move_Duc    HEX        0100,3200,0000       ; Go Duck
GPIAM_A_Move_Jum    HEX        0100,3800,0000       ; Jump / Go Up (From Duck)
GPIAM_A_Move_Pro    HEX        0100,3401,0000       ; Protect
*----
GPIAM_R_Move_Wal    HEX        0100,3600,0000       ; Walk Right
GPIAM_R_Move_FlK    HEX        0100,3802,0000       ; Flight Kick Right
GPIAM_R_Move_Som    HEX        0100,3900,0000       ; Sommersault Right
GPIAM_R_Move_StB    HEX        0100,3400,0000       ; Step Back
GPIAM_R_Move_SoB    HEX        0100,3700,0000       ; Sommersault Back
*-
GPIAM_R_Move_HiP    HEX        0100,3901,0000       ; High Punch Right
GPIAM_R_Move_MiP    HEX        0100,3601,0000       ; Middle Punch Right
GPIAM_R_Move_DuP    HEX        0200,3200,3601,0000  ; Duck + Punch Right
*-
GPIAM_R_Move_HiK    HEX        0100,3902,0000       ; High Kick Right
GPIAM_R_Move_MiK    HEX        0100,3602,0000       ; Middle Kick Right
GPIAM_R_Move_LoK    HEX        0100,3302,0000       ; Low Kick Right
GPIAM_R_Move_BaK    HEX        0100,3702,0000       ; Back Kick Right
GPIAM_R_Move_DuK    HEX        0200,3200,3602,0000  ; Duck + Kick Right
GPIAM_R_Move_DBK    HEX        0100,3402,0000       ; Duck Back Kick Right
*-
GPIAM_R_Move_Cx1    HEX        0300,3400,3400,3902,0000       ; Back + Back + High Kick
GPIAM_R_Move_Cx2    HEX        0300,3400,3602,3400,0000       ; Back + Middle Kick + Back
GPIAM_R_Move_Cx3    HEX        0400,3400,3302,3400,3400,0000  ; Back + Low Kick + Back + Back
GPIAM_R_Move_Cx4    HEX        0300,3400,3901,3400,0000       ; Back + High Punch + Back
GPIAM_R_Move_Cx5    HEX        0300,3400,3601,3400,0000       ; Back + Middle Punch + Back
GPIAM_R_Move_Cx6    HEX        0300,3400,3401,3400,0000       ; Back + Protect + Back
GPIAM_R_Move_Cx7    HEX        0300,3400,3200,3601,0000       ; Back + Duck + Duck Punch
GPIAM_R_Move_Cx8    HEX        0400,3400,3400,3400,3802,0000  ; Back + Back + Back + Flight Kick
*-
GPIAM_R_Move_CxA    HEX        0300,3902,3600,3901,0000       ; High Kick + Walk + High Punch
GPIAM_R_Move_CxB    HEX        0300,3602,3600,3601,0000       ; Middle Kick + Walk + Middle Punch
GPIAM_R_Move_CxC    HEX        0300,3600,3601,3400,0000       ; Walk + Middle Punch + Back
GPIAM_R_Move_CxD    HEX        0300,3302,3600,3901,0000       ; Low Kick + Walk + High Punch
GPIAM_R_Move_CxE    HEX        0300,3602,3600,3600,0000       ; Middle Kick + Walk + Walk
GPIAM_R_Move_CxF    HEX        0300,3900,3503,3402,0000       ; Front Sommersault + Change Direction + Middle Kick
GPIAM_R_Move_CxG    HEX        0300,3802,3503,3702,0000       ; Fligh Kick + Change Direction + High Kick
GPIAM_R_Move_CxH    HEX        0300,3302,3602,3902,0000       ; Low Kick + Middle Kick + High Kick
*----                                
GPIAM_L_Move_Wal    HEX        0100,3400,0000       ; Walk Left
GPIAM_L_Move_FlK    HEX        0100,3802,0000       ; Flight Kick Left
GPIAM_L_Move_Som    HEX        0100,3700,0000       ; Sommersault Left
GPIAM_L_Move_StB    HEX        0100,3600,0000       ; Step Back
GPIAM_L_Move_SoB    HEX        0100,3900,0000       ; Sommersault Back
*-
GPIAM_L_Move_HiP    HEX        0100,3701,0000       ; High Punch Left
GPIAM_L_Move_MiP    HEX        0100,3401,0000       ; Middle Punch Left
GPIAM_L_Move_DuP    HEX        0200,3200,3401,0000  ; Duck + Punch Left
*-
GPIAM_L_Move_HiK    HEX        0100,3702,0000       ; High Kick Left
GPIAM_L_Move_MiK    HEX        0100,3402,0000       ; Middle Kick Left
GPIAM_L_Move_LoK    HEX        0100,3102,0000       ; Low Kick Left
GPIAM_L_Move_BaK    HEX        0100,3902,0000       ; Back Kick Left
GPIAM_L_Move_DuK    HEX        0200,3200,3402,0000  ; Duck + Kick Left
GPIAM_L_Move_DBK    HEX        0100,3602,0000       ; Duck Back Kick Left
*-
GPIAM_L_Move_Cx1    HEX        0300,3600,3600,3702,0000       ; Back + Back + High Kick
GPIAM_L_Move_Cx2    HEX        0300,3600,3402,3600,0000       ; Back + Middle Kick + Back
GPIAM_L_Move_Cx3    HEX        0400,3600,3102,3600,3600,0000  ; Back + Low Kick + Back + Back
GPIAM_L_Move_Cx4    HEX        0300,3600,3701,3600,0000       ; Back + High Punch + Back
GPIAM_L_Move_Cx5    HEX        0300,3600,3401,3600,0000       ; Back + Middle Punch + Back
GPIAM_L_Move_Cx6    HEX        0300,3600,3601,3600,0000       ; Back + Protect + Back
GPIAM_L_Move_Cx7    HEX        0300,3600,3200,3401,0000       ; Back + Duck + Duck Punch
GPIAM_L_Move_Cx8    HEX        0400,3600,3600,3600,3802,0000  ; Back + Back + Back + Flight Kick
*-
GPIAM_L_Move_CxA    HEX        0300,3702,3400,3701,0000       ; High Kick + Walk + High Punch
GPIAM_L_Move_CxB    HEX        0300,3402,3400,3401,0000       ; Middle Kick + Walk + Middle Punch
GPIAM_L_Move_CxC    HEX        0300,3400,3401,3600,0000       ; Walk + Middle Punch + Back
GPIAM_L_Move_CxD    HEX        0300,3102,3400,3701,0000       ; Low Kick + Walk + High Punch
GPIAM_L_Move_CxE    HEX        0300,3402,3400,3400,0000       ; Middle Kick + Walk + Walk
GPIAM_L_Move_CxF    HEX        0300,3700,3503,3602,0000       ; Front Sommersault + Change Direction + Middle Kick
GPIAM_L_Move_CxG    HEX        0300,3802,3503,3902,0000       ; Fligh Kick + Change Direction + High Kick
GPIAM_L_Move_CxH    HEX        0300,3102,3402,3702,0000       ; Low Kick + Middle Kick + High Kick
*--


************************************************************************************************************
*  GetPlayerSequence() :  Get Player Sequence ID (IN:A=SpriteID,IN:Y=SpriteX,IN:X=Input,OUT:A=SequenceID). *
************************************************************************************************************
GetPlayerSequence   STA        GPS_SpriteID
                    STY        GPS_SpriteX
                    STX        GPS_Input            ; 000000KP|0x31-0x39
                    STZ        GPS_SequenceID       ; Init : No Move
*--
                    LDX        #$0000               ; Search for Sprite ID #0
GPS_SearchSpriteID  LDA        GPS_TabSpriteID,X
                    BEQ        GPS_SequenceMove
                    CMP        GPS_SpriteID
                    BEQ        GPS_SearchButton
                    INX
                    INX
                    BRA        GPS_SearchSpriteID
*--
GPS_SearchButton    LDA        GPS_Input            ; Search For Button
                    AND        #$0300               ; 000000KP|00000000
                    BEQ        GPS_SearchButton00   ; No Button
                    CMP        #$0300               ; Button #0 + #1
                    BEQ        GPS_SearchButton11
                    CMP        #$0100               ; Button #0 : Puch
                    BEQ        GPS_SearchButton10
GPS_SearchButton01  LDA        #GPS_TabMove_01      ; Button #1 : Kick
                    BRA        GPS_SearchTab
GPS_SearchButton10  LDA        #GPS_TabMove_10      ; Button #0 : Punch
                    BRA        GPS_SearchTab
GPS_SearchButton11  LDA        #GPS_TabMove_11      ; Button #0 + #1
                    BRA        GPS_SearchTab
GPS_SearchButton00  LDA        #GPS_TabMove_00      ; No Button
*--
GPS_SearchTab       STA        GPS_SearchTab_1+1    ; Search Tab
GPS_SearchTab_1     LDA        $AAAA,X              ; GPM_TabMove_XX
                    STA        GPS_SearchTab_2+1
                    LDA        GPS_Input
                    AND        #$00FF               ; 0x31-0x39
                    SEC
                    SBC        #$0031               ; 0-8
                    ASL
                    TAX                    
GPS_SearchTab_2     LDA        $AAAA,X              ; GPM_Tab_XX_YYY                    
                    STA        GPS_SearchSequence_1+1
*--
GPS_SearchSequence  LDY        #$0000               ; Search Sequence ID
GPS_SearchSequence_1 LDA       $AAAA,Y              ; PM_XX_YYY_[1-9]
                    BEQ        GPS_SequenceNoMove   ; (SequenceID == 0) => No Move
                    STA        GPS_SequenceID
                    ASL                             ; Check Screen Borders
                    TAX
                    LDA        TAB_SEQUENCE_MOVE,X  ; Check if this 
                    BEQ        GPS_SequenceMove     ; (Sequence Move == 0) => Authorized
                    LDA        TAB_SEQUENCE_WIDTH,X
                    BEQ        GPS_SequenceMove     ; (Sequence Width == 0) => Authorized
                    LDX        GPS_SpriteX
                    JSR        AddSignedValues      ; SpriteX + SequenceWidth
                    ASL
                    BCS        GPS_SearchSequence_2 ; Negative => Out of Screen
                    LSR
                    CMP        GPS_ScreenLeftX      ; Left Border
                    BLT        GPS_SearchSequence_2
                    CMP        GPS_ScreenRightX     ; Right Border + 1
                    BLT        GPS_SequenceMove
GPS_SearchSequence_2 INY                            ; Next Sequence
                    INY
                    BRA        GPS_SearchSequence_1
*--
GPS_SequenceNoMove  STZ        GPS_SequenceID       ; (0000 = NoMove)
GPS_SequenceMove    LDA        GPS_SequenceID       ; New Sequence ID 
                    RTS               
                    
*----------   GetPlayerSequence Variables   ---------
GPS_SpriteID        HEX        0000
GPS_SpriteX         HEX        0000
GPS_Input           HEX        0000                 ; 000000KP|0x31-0x39
GPS_SequenceID      HEX        0000                 ; Returned Sequence ID
GPS_ScreenLeftX     HEX        0D00                 ; 26                 
GPS_ScreenRightX    HEX        8000                 ; 256
GPS_TabSpriteID     HEX        0100,2200,4100,6200  ; UP   : PBR PBL PRR PRL
                    HEX        1A00,3B00,5A00,7B00  ; DUCK : PBR PBL PRR PRL
                    HEX        0000
*--
GPS_TabMove_00      DA         GPS_Tab_00_UBR,GPS_Tab_00_UBL,GPS_Tab_00_URR,GPS_Tab_00_URL  ; NoButton   Player Up   Blue/Red Right/Left
                    DA         GPS_Tab_00_DBR,GPS_Tab_00_DBL,GPS_Tab_00_DRR,GPS_Tab_00_DRL  ; NoButton   Player Duck Blue/Red Right/Left
GPS_TabMove_10      DA         GPS_Tab_10_UBR,GPS_Tab_10_UBL,GPS_Tab_10_URR,GPS_Tab_10_URL  ; Button 0   Player Up   Blue/Red Right/Left
                    DA         GPS_Tab_10_DBR,GPS_Tab_10_DBL,GPS_Tab_10_DRR,GPS_Tab_10_DRL  ; Button 0   Player Duck Blue/Red Right/Left
GPS_TabMove_01      DA         GPS_Tab_01_UBR,GPS_Tab_01_UBL,GPS_Tab_01_URR,GPS_Tab_01_URL  ; Button 1   Player Up   Blue/Red Right/Left
                    DA         GPS_Tab_01_DBR,GPS_Tab_01_DBL,GPS_Tab_01_DRR,GPS_Tab_01_DRL  ; Button 1   Player Duck Blue/Red Right/Left
GPS_TabMove_11      DA         GPS_Tab_11_UBR,GPS_Tab_11_UBL,GPS_Tab_11_URR,GPS_Tab_11_URL  ; Button 0+1 Player Up   Blue/Red Right/Left
                    DA         GPS_Tab_11_DBR,GPS_Tab_11_DBL,GPS_Tab_11_DRR,GPS_Tab_11_DRL  ; Button 0+1 Player Duck Blue/Red Right/Left
*--
GPS_XX_XXX_X        HEX        0000 ; No Sequence available
*-- No Button
GPS_Tab_00_UBR      DA         GPS_XX_XXX_X,GPS_00_UBR_2,GPS_XX_XXX_X,GPS_00_UBR_4,GPS_XX_XXX_X,GPS_00_UBR_6,GPS_00_UBR_7,GPS_00_UBR_8,GPS_00_UBR_9 ; NoButton Player Up Blue Right
GPS_Tab_00_UBL      DA         GPS_XX_XXX_X,GPS_00_UBL_2,GPS_XX_XXX_X,GPS_00_UBL_4,GPS_XX_XXX_X,GPS_00_UBL_6,GPS_00_UBL_7,GPS_00_UBL_8,GPS_00_UBL_9 ; NoButton Player Up Blue Left
GPS_Tab_00_URR      DA         GPS_XX_XXX_X,GPS_00_URR_2,GPS_XX_XXX_X,GPS_00_URR_4,GPS_XX_XXX_X,GPS_00_URR_6,GPS_00_URR_7,GPS_00_URR_8,GPS_00_URR_9 ; NoButton Player Up Red Right
GPS_Tab_00_URL      DA         GPS_XX_XXX_X,GPS_00_URL_2,GPS_XX_XXX_X,GPS_00_URL_4,GPS_XX_XXX_X,GPS_00_URL_6,GPS_00_URL_7,GPS_00_URL_8,GPS_00_URL_9 ; NoButton Player Up Red Left
GPS_Tab_00_DBR      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_00_DBR_8,GPS_XX_XXX_X ; NoButton Player Duck Blue Right
GPS_Tab_00_DBL      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_00_DBL_8,GPS_XX_XXX_X ; NoButton Player Duck Blue Left
GPS_Tab_00_DRR      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_00_DRR_8,GPS_XX_XXX_X ; NoButton Player Duck Red Right
GPS_Tab_00_DRL      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_00_DRL_8,GPS_XX_XXX_X ; NoButton Player Duck Red Left
*- Player Up Blue Right
GPS_00_UBR_2        HEX        0300,0000            ; Player Blue Right : Go Down
GPS_00_UBR_4        HEX        0200,0000            ; Player Blue Right : Walk Left
GPS_00_UBR_6        HEX        0100,0000            ; Player Blue Right : Walk Right
GPS_00_UBR_7        HEX        0A00,0B00,0C00,0000  ; Player Blue Right : Jump Roll Back Long / Short / Place
GPS_00_UBR_8        HEX        0600,0000            ; Player Blue Right : Jump Up
GPS_00_UBR_9        HEX        0700,0800,0900,0000  ; Player Blue Right : Jump Roll Front Long / Short / Place
*- Player Up Blue Left
GPS_00_UBL_2        HEX        2000,0000            ; Player Blue Left : Go Down
GPS_00_UBL_4        HEX        1E00,0000            ; Player Blue Left : Walk Left
GPS_00_UBL_6        HEX        1F00,0000            ; Player Blue Left : Walk Right
GPS_00_UBL_7        HEX        2400,2500,2600,0000  ; Player Blue Left : Jump Roll Front Long / Short / Place
GPS_00_UBL_8        HEX        2300,0000            ; Player Blue Left : Jump Up
GPS_00_UBL_9        HEX        2700,2800,2900,0000  ; Player Blue Left : Jump Roll Back Long / Short / Place
*- Player Up Red Right
GPS_00_URR_2        HEX        3D00,0000            ; Player Red Right : Go Down
GPS_00_URR_4        HEX        3C00,0000            ; Player Red Right : Walk Left
GPS_00_URR_6        HEX        3B00,0000            ; Player Red Right : Walk Right
GPS_00_URR_7        HEX        4400,4500,4600,0000  ; Player Red Right : Jump Roll Back Long / Short / Place
GPS_00_URR_8        HEX        4000,0000            ; Player Red Right : Jump Up
GPS_00_URR_9        HEX        4100,4200,4300,0000  ; Player Red Right : Jump Roll Front Long / Short / Place
*- Player Up Red Left
GPS_00_URL_2        HEX        5A00,0000            ; Player Red Left : Go Down
GPS_00_URL_4        HEX        5800,0000            ; Player Red Left : Walk Left
GPS_00_URL_6        HEX        5900,0000            ; Player Red Left : Walk Right
GPS_00_URL_7        HEX        5E00,5F00,6000,0000  ; Player Red Left : Jump Roll Front Long / Short / Place
GPS_00_URL_8        HEX        5D00,0000            ; Player Red Left : Jump Up
GPS_00_URL_9        HEX        6100,6200,6300,0000  ; Player Red Left : Jump Roll Back Long / Short / Place
*- Player Duck Blue Right
GPS_00_DBR_8        HEX        0400,0000            ; Player Blue Right : Go Up from Duck
*- Player Duck Blue Left
GPS_00_DBL_8        HEX        2100,0000            ; Player Blue Left : Go Up from Duck
*- Player Duck Red Right
GPS_00_DRR_8        HEX        3E00,0000            ; Player Red Right : Go Up from Duck
*- Player Duck Red Left
GPS_00_DRL_8        HEX        5B00,0000            ; Player Red Left : Go Up from Duck
*-- Button #0
GPS_Tab_10_UBR      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_10_UBR_4,GPS_XX_XXX_X,GPS_10_UBR_6,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_10_UBR_9 ; Button 0 Player Up Blue Right
GPS_Tab_10_UBL      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_10_UBL_4,GPS_XX_XXX_X,GPS_10_UBL_6,GPS_10_UBL_7,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 0 Player Up Blue Left
GPS_Tab_10_URR      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_10_URR_4,GPS_XX_XXX_X,GPS_10_URR_6,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_10_URR_9 ; Button 0 Player Up Red Right
GPS_Tab_10_URL      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_10_URL_4,GPS_XX_XXX_X,GPS_10_URL_6,GPS_10_URL_7,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 0 Player Up Red Left
GPS_Tab_10_DBR      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_10_DBR_6,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 0 Player Duck Blue Right
GPS_Tab_10_DBL      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_10_DBL_4,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 0 Player Duck Blue Left
GPS_Tab_10_DRR      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_10_DRR_6,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 0 Player Duck Red Right
GPS_Tab_10_DRL      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_10_DRL_4,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 0 Player Duck Red Left
*- Player Up Blue Right + Button #0
GPS_10_UBR_4        HEX        0D00,0000            ; Player Blue Right : Protect
GPS_10_UBR_6        HEX        0F00,0000            ; Player Blue Right : Middle Punch
GPS_10_UBR_9        HEX        0E00,0000            ; Player Blue Right : High Punch
*- Player Up Blue Left + Button #0
GPS_10_UBL_4        HEX        2C00,0000            ; Player Blue Left : Middle Punch
GPS_10_UBL_6        HEX        2A00,0000            ; Player Blue Left : Protect
GPS_10_UBL_7        HEX        2B00,0000            ; Player Blue Left : High Punch
*- Player Up Red Right + Button #0
GPS_10_URR_4        HEX        4700,0000            ; Player Red Right : Protect
GPS_10_URR_6        HEX        4900,0000            ; Player Red Right : Middle Punch
GPS_10_URR_9        HEX        4800,0000            ; Player Red Right : High Punch
*- Player Up Red Left + Button #0
GPS_10_URL_4        HEX        6600,0000            ; Player Red Left : Middle Punch
GPS_10_URL_6        HEX        6400,0000            ; Player Red Left : Protect
GPS_10_URL_7        HEX        6500,0000            ; Player Red Left : High Punch
*- Player Duck Blue Right + Button #0
GPS_10_DBR_6        HEX        1700,0000            ; Player Blue Right : Duck Punch
*- Player Duck Blue Left + Button #0
GPS_10_DBL_4        HEX        3400,0000            ; Player Blue Left : Duck Punch
*- Player Duck Red Right + Button #0
GPS_10_DRR_6        HEX        5100,0000            ; Player Red Right : Duck Punch
*- Player Duck Red Left + Button #0
GPS_10_DRL_4        HEX        6E00,0000            ; Player Red Left : Duck Punch
*-- Button #1
GPS_Tab_01_UBR      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_01_UBR_3,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_01_UBR_6,GPS_01_UBR_7,GPS_01_UBR_8,GPS_01_UBR_9 ; Button 1 Player Up Blue Right
GPS_Tab_01_UBL      DA         GPS_01_UBL_1,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_01_UBL_4,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_01_UBL_7,GPS_01_UBL_8,GPS_01_UBL_9 ; Button 1 Player Up Blue Left
GPS_Tab_01_URR      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_01_URR_3,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_01_URR_6,GPS_01_URR_7,GPS_01_URR_8,GPS_01_URR_9 ; Button 1 Player Up Red Right
GPS_Tab_01_URL      DA         GPS_01_URL_1,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_01_URL_4,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_01_URL_7,GPS_01_URL_8,GPS_01_URL_9 ; Button 1 Player Up Red Left
GPS_Tab_01_DBR      DA         GPS_01_DBR_1,GPS_XX_XXX_X,GPS_01_DBR_3,GPS_01_DBR_4,GPS_XX_XXX_X,GPS_01_DBR_6,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 1 Player Duck Blue Right
GPS_Tab_01_DBL      DA         GPS_01_DBL_1,GPS_XX_XXX_X,GPS_01_DBL_3,GPS_01_DBL_4,GPS_XX_XXX_X,GPS_01_DBL_6,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 1 Player Duck Blue Left
GPS_Tab_01_DRR      DA         GPS_01_DRR_1,GPS_XX_XXX_X,GPS_01_DRR_3,GPS_01_DRR_4,GPS_XX_XXX_X,GPS_01_DRR_6,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 1 Player Duck Red Right
GPS_Tab_01_DRL      DA         GPS_01_DRL_1,GPS_XX_XXX_X,GPS_01_DRL_3,GPS_01_DRL_4,GPS_XX_XXX_X,GPS_01_DRL_6,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 1 Player Duck Red Left
*- Player Up Blue Right + Button #1
GPS_01_UBR_3        HEX        1200,0000            ; Player Blue Right : Low Kick
GPS_01_UBR_6        HEX        1100,0000            ; Player Blue Right : Middle Kick
GPS_01_UBR_7        HEX        1300,0000            ; Player Blue Right : Back High Kick
GPS_01_UBR_8        HEX        1400,1500,1600,0000  ; Player Blue Right : Flight Kick Long / Short / Place
GPS_01_UBR_9        HEX        1000,0000            ; Player Blue Right : High Kick
*- Player Up Blue Left + Button #1
GPS_01_UBL_1        HEX        2F00,0000            ; Player Blue Left : Low Kick
GPS_01_UBL_4        HEX        2E00,0000            ; Player Blue Left : Middle Kick
GPS_01_UBL_7        HEX        2D00,0000            ; Player Blue Left : High Kick
GPS_01_UBL_8        HEX        3100,3200,3300,0000  ; Player Blue Left : Flight Kick Long / Short / Place
GPS_01_UBL_9        HEX        3000,0000            ; Player Blue Left : Back High Kick
*- Player Up Red Right + Button #1
GPS_01_URR_3        HEX        4C00,0000            ; Player Red Right : Low Kick
GPS_01_URR_6        HEX        4B00,0000            ; Player Red Right : Middle Kick
GPS_01_URR_7        HEX        4D00,0000            ; Player Red Right : Back High Kick
GPS_01_URR_8        HEX        4E00,4F00,5000,0000  ; Player Red Right : Flight Kick Long / Short / Place
GPS_01_URR_9        HEX        4A00,0000            ; Player Red Right : High Kick
*- Player Up Red Left + Button #1
GPS_01_URL_1        HEX        6900,0000            ; Player Red Left : Low Kick
GPS_01_URL_4        HEX        6800,0000            ; Player Red Left : Middle Kick
GPS_01_URL_7        HEX        6700,0000            ; Player Red Left : High Kick
GPS_01_URL_8        HEX        6B00,6C00,6D00,0000  ; Player Red Left : Flight Kick Long / Short / Place
GPS_01_URL_9        HEX        6A00,0000            ; Player Red Left : Back High Kick
*- Player Duck Blue Right + Button #1                       
GPS_01_DBR_1        HEX        1900,0000            ; Player Blue Right : Duck Back Kick
GPS_01_DBR_3        HEX        1800,0000            ; Player Blue Right : Duck Kick
GPS_01_DBR_4        HEX        1900,0000            ; Player Blue Right : Duck Back Kick
GPS_01_DBR_6        HEX        1800,0000            ; Player Blue Right : Duck Kick
*- Player Duck Blue Left + Button #1
GPS_01_DBL_1        HEX        3500,0000            ; Player Blue Left : Duck Kick
GPS_01_DBL_3        HEX        3600,0000            ; Player Blue Left : Duck Back Kick
GPS_01_DBL_4        HEX        3500,0000            ; Player Blue Left : Duck Kick
GPS_01_DBL_6        HEX        3600,0000            ; Player Blue Left : Duck Back Kick
*- Player Duck Red Right + Button #1
GPS_01_DRR_1        HEX        5300,0000            ; Player Red Right : Duck Back Kick
GPS_01_DRR_3        HEX        5200,0000            ; Player Red Right : Duck Kick
GPS_01_DRR_4        HEX        5300,0000            ; Player Red Right : Duck Back Kick
GPS_01_DRR_6        HEX        5200,0000            ; Player Red Right : Duck Kick
*- Player Duck Red Left + Button #1
GPS_01_DRL_1        HEX        6F00,0000            ; Player Red Left : Duck Kick
GPS_01_DRL_3        HEX        7000,0000            ; Player Red Left : Duck Back Kick
GPS_01_DRL_4        HEX        6F00,0000            ; Player Red Left : Duck Kick
GPS_01_DRL_6        HEX        7000,0000            ; Player Red Left : Duck Back Kick
*-- Buttons #0 + #1
GPS_Tab_11_UBR      DA         GPS_11_UBR_A,GPS_11_UBR_A,GPS_11_UBR_A,GPS_11_UBR_A,GPS_11_UBR_A,GPS_11_UBR_A,GPS_11_UBR_A,GPS_11_UBR_A,GPS_11_UBR_A ; Button 0+1 Player Up Blue Right
GPS_Tab_11_UBL      DA         GPS_11_UBL_A,GPS_11_UBL_A,GPS_11_UBL_A,GPS_11_UBL_A,GPS_11_UBL_A,GPS_11_UBL_A,GPS_11_UBL_A,GPS_11_UBL_A,GPS_11_UBL_A ; Button 0+1 Player Up Blue Left
GPS_Tab_11_URR      DA         GPS_11_URR_A,GPS_11_URR_A,GPS_11_URR_A,GPS_11_URR_A,GPS_11_URR_A,GPS_11_URR_A,GPS_11_URR_A,GPS_11_URR_A,GPS_11_URR_A ; Button 0+1 Player Up Red Right
GPS_Tab_11_URL      DA         GPS_11_URL_A,GPS_11_URL_A,GPS_11_URL_A,GPS_11_URL_A,GPS_11_URL_A,GPS_11_URL_A,GPS_11_URL_A,GPS_11_URL_A,GPS_11_URL_A ; Button 0+1 Player Up Red Left
GPS_Tab_11_DBR      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 0+1 Player Duck Blue Right
GPS_Tab_11_DBL      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 0+1 Player Duck Blue Left
GPS_Tab_11_DRR      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 0+1 Player Duck Red Right
GPS_Tab_11_DRL      DA         GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X,GPS_XX_XXX_X ; Button 0+1 Player Duck Red Left   
*- Player Up Blue Right + Buttons #0 + #1
GPS_11_UBR_A        HEX        0500,0000            ; Player Blue Right : Turn Back
*- Player Up Blue Left + Buttons #0 + #1
GPS_11_UBL_A        HEX        2200,0000            ; Player Blue Left : Turn Back
*- Player Up Red Right + Buttons #0 + #1
GPS_11_URR_A        HEX        3F00,0000            ; Player Red Right : Turn Back
*- Player Up Red Left + Buttons #0 + #1
GPS_11_URL_A        HEX        5C00,0000            ; Player Red Left : Turn Back
*- Player Duck Blue Right + Buttons #0 + #1
*- Player Duck Blue Left + Buttons #0 + #1
*- Player Duck Red Right + Buttons #0 + #1
*- Player Duck Red Left + Buttons #0 + #1
     
********************************************************************************