******************************************************************************************
*                                                                                        *
*  AppleIIgsKarate.Graphic.s :  Library for Graphic Handling (Numbers + Character Set).  *
*                                                                                        *
******************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI  *  Brutal Deluxe Software  *  April 2022  *
******************************************************************************************


***********************************************************************************
*  DisplayString() :  Display a String on the Screen (IN:X=@String,IN:Y=@Screen). *
***********************************************************************************
DisplayString       ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--
                    LDA        $04,S                ; Get Caller's Bank  A=??BB
                    XBA
                    STA        DS_String+2          ; String Bank
                    STX        DS_String+1          ; String Address
                    STY        DS_Screen+1          ; Screen Address
                    STZ        DS_Index+1
*--
DS_Index            LDX        #$0000                     
DS_String           LDAL       $BBAAAA,X              ; String Data
                    AND        #$00FF
                    STA        DS_Char_2+1
                    BEQ        DS_End               ; End of String
                    INC        DS_Index+1
*--
                    LDX        #$0000               ; Search character into 
DS_Char_1           LDA        Alpha_ASCII_Tab,X    ; authorized characters list
                    AND        #$00FF
                    BEQ        DS_Index             ; Unknown character
DS_Char_2           CMP        #$AAAA
                    BEQ        DS_Display
                    INX
                    BRA        DS_Char_1
*--                    
DS_Display          TXA                             ; Character Bitmap Address
                    ASL
                    TAX
                    LDA        Alpha_Bitmap_Tab,X
                    TAX                             ; Bitmap Address
DS_Screen           LDY        #$AAAA               ; Screen Address
                    LDA        #$0208               ; Width4 / Heigh
                    JSL        CopyOneBitmap        ; Copy 8*8 Bitmap
*--
                    LDA        DS_Screen+1          ; New Screen Address
                    CLC
                    ADC        #$0004
                    STA        DS_Screen+1
                    BRA        DS_Index
*--   
DS_End              PLB
                    RTL


******************************************************************************************
*  CopyOneBitmap() :  Copy one Bitmap to Screen (IN:A=Width4|Heigh,IN:X=@Src,IN:Y=@Dst). *
******************************************************************************************
CopyOneBitmap       ENT
                    PHB
                    PHK                             ; Current Bank = Data Bank
                    PLB
*--                           
                    STX        COB_Data+1           ; X = Src @
                    STY        COB_Screen+1         ; Y = Dst @
                    TAX                             ; A = Width4/Heigh
                    AND        #$00FF               ; Heigh
                    STA        COB_Heigh+1
                    TXA
                    XBA
                    AND        #$00FF               ; Width4
                    STA        COB_Width+1
                    LDA        #$00A0
                    SEC
                    SBC        COB_Width+1
                    SEC
                    SBC        COB_Width+1
                    STA        COB_Offset+1
*--                                                    
                    LDY        #$0000               ; Copy Bitmap Data
                    LDX        #$0000
COB_Data            LDA        $AAAA                ; Number Pixel
COB_Screen          STAL       $012000              ; Screen Address
                    INC        COB_Data+1
                    INC        COB_Data+1
                    INC        COB_Screen+1
                    INC        COB_Screen+1                    
                    INX
COB_Width           CPX        #$AAAA
                    BNE        COB_Data
                    LDA        COB_Screen+1
                    CLC
COB_Offset          ADC        #$00A0               ; #$A0 - Width
                    STA        COB_Screen+1
                    LDX        #$0000
                    INY
COB_Heigh           CPY        #$AAAA
                    BNE        COB_Data
*--
                    PLB
                    RTL


***********************************
*  Alphabet :  0-9 A-Z Space Dot. *
***********************************

Alpha_ASCII_Tab     ASC        ' 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ.:',00

Alpha_Bitmap_Tab    DA         Alpha_Space,Alpha_0,Alpha_1,Alpha_2,Alpha_3,Alpha_4,Alpha_5,Alpha_6
                    DA         Alpha_7,Alpha_8,Alpha_9,Alpha_A,Alpha_B,Alpha_C,Alpha_D,Alpha_E
                    DA         Alpha_F,Alpha_G,Alpha_H,Alpha_I,Alpha_J,Alpha_K,Alpha_L,Alpha_M
                    DA         Alpha_N,Alpha_O,Alpha_P,Alpha_Q,Alpha_R,Alpha_S,Alpha_T,Alpha_U
                    DA         Alpha_V,Alpha_W,Alpha_X,Alpha_Y,Alpha_Z,Alpha_Dot,Alpha_DoubleDot

Score_Bitmap_Tab    ENT
                    DA         Score_0,Score_1,Score_2,Score_3,Score_4,Score_5,Score_6,Score_7,Score_8,Score_9


Alpha_Space         HEX        00000000             ; Space
                    HEX        00000000
                    HEX        00000000
                    HEX        00000000
                    HEX        00000000
                    HEX        00000000
                    HEX        00000000
                    HEX        00000000

Alpha_0             HEX        00000000             ; 0
                    HEX        00FFFF00
                    HEX        0F000FF0
                    HEX        0F0000F0
                    HEX        0FF000F0
                    HEX        0FF000F0
                    HEX        0FFF0FF0
                    HEX        00FFFF00
                    
Alpha_1             HEX        00000000             ; 1
                    HEX        000FF000
                    HEX        00FFF000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        00FFFF00
                    
Alpha_2             HEX        00000000             ; 2
                    HEX        00FFFF00
                    HEX        0F000FF0
                    HEX        00000FF0
                    HEX        0000FF00
                    HEX        000FF000
                    HEX        00FF00F0
                    HEX        0FFFFFF0
                    
Alpha_3             HEX        00000000             ; 3
                    HEX        00FFFF00
                    HEX        0F000FF0
                    HEX        00000FF0
                    HEX        000FFF00
                    HEX        00000FF0
                    HEX        0F000FF0
                    HEX        00FFFF00
                    
Alpha_4             HEX        00000000             ; 4
                    HEX        0000FF00
                    HEX        0000FF00
                    HEX        00F0FF00
                    HEX        00F0FF00
                    HEX        0F00FF00
                    HEX        0FFFFFF0
                    HEX        0000FF00
                    
Alpha_5             HEX        00000000             ; 5
                    HEX        0FFFFFF0
                    HEX        0FF000F0
                    HEX        0FF00000
                    HEX        000FFF00
                    HEX        00000FF0
                    HEX        0F000FF0
                    HEX        00FFFF00
                    
Alpha_6             HEX        00000000             ; 6      
                    HEX        000F0000
                    HEX        00F00000
                    HEX        0FF0FF00
                    HEX        0FFFFFF0
                    HEX        0FF00FF0
                    HEX        0FFF00F0
                    HEX        00FFFF00
                    
Alpha_7             HEX        00000000             ; 7
                    HEX        0FFFFFF0
                    HEX        0F00FF00
                    HEX        0000F000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        000F0000
                    
Alpha_8             HEX        00000000             ; 8
                    HEX        00FFFF00
                    HEX        0FF00FF0
                    HEX        0FF000F0
                    HEX        00FFFF00
                    HEX        0F00FFF0
                    HEX        0FF00FF0
                    HEX        00FFFF00
                    
Alpha_9             HEX        00000000             ; 9      
                    HEX        00FFFF00
                    HEX        0F00FFF0
                    HEX        0FF00FF0
                    HEX        0FFFFFF0
                    HEX        00FF0FF0
                    HEX        00000F00
                    HEX        0000F000
                    
Alpha_A             HEX        0FFFFF00             ; A 
                    HEX        F0000FF0
                    HEX        0FF00FF0
                    HEX        0FFFFFF0
                    HEX        0FF00FF0
                    HEX        0FF00FF0
                    HEX        0FF00FF0
                    HEX        FFFF0FF0
                    
Alpha_B             HEX        0FFFFF00             ; B
                    HEX        F0000FF0
                    HEX        0FF00FF0
                    HEX        0FFFFF00
                    HEX        0FF00FF0
                    HEX        0FF000F0
                    HEX        0FF00FF0
                    HEX        FFFFFF00
                    
Alpha_C             HEX        00FFFF00             ; C
                    HEX        0FF000F0
                    HEX        FF000000
                    HEX        FF000000
                    HEX        FF000000
                    HEX        FF000000
                    HEX        FFF000F0
                    HEX        0FFFFF00
                    
Alpha_D             HEX        0FFFFF00             ; D
                    HEX        F0000FF0
                    HEX        0FF000F0
                    HEX        0FF000F0
                    HEX        0FF000F0
                    HEX        0FF000F0
                    HEX        0FF00FF0
                    HEX        FFFFFF00
                    
Alpha_E             HEX        FFFFFFF0             ; E
                    HEX        0FF000F0
                    HEX        0FF0FF00
                    HEX        0FFF0F00
                    HEX        0FF00000
                    HEX        0FF000F0
                    HEX        0FF00FF0
                    HEX        FFFFFFF0
                    
Alpha_F             HEX        FFFFFFF0             ; F
                    HEX        0FF000F0
                    HEX        0FF0FF00
                    HEX        0FFF0F00
                    HEX        0FF00000
                    HEX        0FF00000
                    HEX        0FF00000
                    HEX        FFFF0000
                    
Alpha_G             HEX        0FFFFF00             ; G
                    HEX        FFF000F0
                    HEX        FF000000
                    HEX        FF00FFF0
                    HEX        FF0000F0
                    HEX        FF000FF0
                    HEX        FFF00FF0
                    HEX        0FFFFF00
                    
Alpha_H             HEX        FFF00FF0             ; H
                    HEX        0FF00FF0
                    HEX        0FF00FF0
                    HEX        0FFFFFF0
                    HEX        0FF00FF0
                    HEX        0FF00FF0
                    HEX        0FF00FF0
                    HEX        FFF00FF0

Alpha_I             HEX        00FFFF00             ; I
                    HEX        000FF000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        00FFFF00
                    
Alpha_J             HEX        00FFFFF0             ; J
                    HEX        0000FF00
                    HEX        0000FF00
                    HEX        0000FF00
                    HEX        0000FF00
                    HEX        F000FF00
                    HEX        FF0FFF00
                    HEX        0FFFF000

Alpha_K             HEX        FFFF00F0             ; K
                    HEX        0FF000F0
                    HEX        0FF00F00
                    HEX        0FFFF000
                    HEX        0FF0FF00
                    HEX        0FF00FF0
                    HEX        0FF000F0
                    HEX        FFFF0FF0

Alpha_L             HEX        FFFF0000             ; L
                    HEX        0FF00000
                    HEX        0FF00000
                    HEX        0FF00000
                    HEX        0FF00000
                    HEX        0FF000F0
                    HEX        0FF00FF0
                    HEX        FFFFFFF0

Alpha_M             HEX        FF000FF0             ; M
                    HEX        FFF000F0
                    HEX        0FFF0FF0
                    HEX        0FFFFFF0
                    HEX        0FF0F0F0
                    HEX        0FF000F0
                    HEX        0FF000F0
                    HEX        FFFF00F0

Alpha_N             HEX        FF00FFF0             ; N
                    HEX        FFF00F00
                    HEX        0FFF0F00
                    HEX        0FFFFF00
                    HEX        0F0FFF00
                    HEX        0F00FFF0
                    HEX        0F000FF0
                    HEX        FFF00F00

Alpha_O             HEX        00FFFF00             ; O
                    HEX        0F00FFF0
                    HEX        FF0000F0
                    HEX        FF0000F0
                    HEX        FF0000F0
                    HEX        FF0000F0
                    HEX        FFF00F00
                    HEX        0FFFF000

Alpha_P             HEX        FFFFFF00             ; P
                    HEX        0FF00FF0
                    HEX        0FF000F0
                    HEX        0FFF00F0
                    HEX        0FF0FF00
                    HEX        0FF00000
                    HEX        0FF00000
                    HEX        FFFF0000

Alpha_Q             HEX        00FFFF00             ; Q
                    HEX        0F00FFF0
                    HEX        FF0000F0
                    HEX        FF0000F0
                    HEX        FF0000F0
                    HEX        FF0FF0F0
                    HEX        FFF00F00
                    HEX        0FFF0FF0

Alpha_R             HEX        0FFFFF00             ; R
                    HEX        F0000FF0
                    HEX        0FF00FF0
                    HEX        0FF0FF00
                    HEX        0FF00FF0
                    HEX        0FF00FF0
                    HEX        0FF00FF0
                    HEX        FFFF0FF0

Alpha_S             HEX        0FFFFF00             ; S
                    HEX        FFF000F0
                    HEX        FF000000
                    HEX        0FFFF000
                    HEX        00FFFF00
                    HEX        00000FF0
                    HEX        F000FFF0
                    HEX        0FFFFF00

Alpha_T             HEX        0FFFFFF0             ; T
                    HEX        0F0FF0F0
                    HEX        000FF000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        00FFFF00

Alpha_U             HEX        FFF00F00             ; U
                    HEX        0FF00FF0
                    HEX        0FF000F0
                    HEX        0FF000F0
                    HEX        0FF000F0
                    HEX        0FF000F0
                    HEX        0FFF0FF0
                    HEX        00FFFF00

Alpha_V             HEX        FFF000F0             ; V
                    HEX        0FF00FF0
                    HEX        0FF00FF0
                    HEX        0FF00FF0
                    HEX        0FF00FF0
                    HEX        00FF0FF0
                    HEX        00FFFFF0
                    HEX        000FF000

Alpha_W             HEX        FFF000F0             ; W
                    HEX        0FF000F0
                    HEX        0FF000F0
                    HEX        0FF0F0F0
                    HEX        0FFFFFF0
                    HEX        0FFFFFF0
                    HEX        0FFF0FF0
                    HEX        0FF000F0

Alpha_X             HEX        0FFF00F0             ; X
                    HEX        FFF000F0
                    HEX        0FFF0F00
                    HEX        00FFF000
                    HEX        000FFF00
                    HEX        00F0FF00
                    HEX        0F000FF0
                    HEX        F0000FF0

Alpha_Y             HEX        0FF00FF0             ; Y
                    HEX        FFF000F0
                    HEX        0FF000F0
                    HEX        00FF0F00
                    HEX        000FF000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        00FFFF00

Alpha_Z             HEX        0FFFFFF0             ; Z
                    HEX        FFFFFF00
                    HEX        F000F000
                    HEX        000F0000
                    HEX        00F00000
                    HEX        0F0000F0
                    HEX        FFFFFFF0
                    HEX        FFFFFF00

Alpha_Dot           HEX        00000000             ; .
                    HEX        00000000
                    HEX        00000000
                    HEX        00000000
                    HEX        00000000
                    HEX        00000000
                    HEX        000FF000
                    HEX        000FF000

Alpha_DoubleDot     HEX        00000000             ; :
                    HEX        00000000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        00000000
                    HEX        000FF000
                    HEX        000FF000
                    HEX        00000000

Score_0             ENT
                    HEX        00000000             ; 0
                    HEX        00222200
                    HEX        02000220
                    HEX        02000020
                    HEX        02200020
                    HEX        02200020
                    HEX        02220220
                    HEX        00222200
                    
Score_1             HEX        00000000             ; 1
                    HEX        00022000
                    HEX        00222000
                    HEX        00022000
                    HEX        00022000
                    HEX        00022000
                    HEX        00022000
                    HEX        00222200
                    
Score_2             HEX        00000000             ; 2
                    HEX        00222200
                    HEX        02000220
                    HEX        00000220
                    HEX        00002200
                    HEX        00022000
                    HEX        00220020
                    HEX        02222220
                    
Score_3             HEX        00000000             ; 3
                    HEX        00222200
                    HEX        02000220
                    HEX        00000220
                    HEX        00022200
                    HEX        00000220
                    HEX        02000220
                    HEX        00222200
                    
Score_4             HEX        00000000             ; 4
                    HEX        00002200
                    HEX        00002200
                    HEX        00202200
                    HEX        00202200
                    HEX        02002200
                    HEX        02222220
                    HEX        00002200
                    
Score_5             HEX        00000000             ; 5
                    HEX        02222220
                    HEX        02200020
                    HEX        02200000
                    HEX        00022200
                    HEX        00000220
                    HEX        02000220
                    HEX        00222200
                    
Score_6             HEX        00000000             ; 6      
                    HEX        00020000
                    HEX        00200000
                    HEX        02202200
                    HEX        02222220
                    HEX        02200220
                    HEX        02220020
                    HEX        00222200
                    
Score_7             HEX        00000000             ; 7
                    HEX        02222220
                    HEX        02002200
                    HEX        00002000
                    HEX        00022000
                    HEX        00022000
                    HEX        00022000
                    HEX        00020000
                    
Score_8             HEX        00000000             ; 8
                    HEX        00222200
                    HEX        02200220
                    HEX        02200020
                    HEX        00222200
                    HEX        02002220
                    HEX        02200220
                    HEX        00222200
                    
Score_9             HEX        00000000             ; 9      
                    HEX        00222200
                    HEX        02002220
                    HEX        02200220
                    HEX        02222220
                    HEX        00220220
                    HEX        00000200
                    HEX        00002000
                    

*************************
*  Boot Logo + Palette. *
*************************

BOOT_PALETTE        HEX        0000,2202,3303,4404,5505,6606,7707,8808,9909,AA0A,BB0B,DD0D,EE0E,FF0F,FF0F,FF0F

BOOT_LOGO           HEX        00,00,00,00,00,00,00,00,00,00,00,00,01,24,46,64,42,10,00,00,00,00,00,00,00,00,00,00,00,00
                    HEX        00,00,00,00,00,00,00,00,00,00,01,7A,CD,DD,DD,DD,DD,DC,A7,20,00,00,00,00,00,00,00,00,00,00
                    HEX        00,00,00,00,00,00,00,00,00,17,CD,DD,DD,DC,BB,BB,CD,DD,DD,DC,71,00,00,00,00,00,00,00,00,00
                    HEX        00,00,00,00,00,00,00,00,2A,DD,DD,B8,41,00,00,00,00,14,8B,DD,DD,A2,00,00,00,00,00,00,00,00
                    HEX        00,00,00,00,00,00,00,19,DD,DC,71,00,00,00,00,00,00,00,00,17,CD,DD,A1,00,00,00,00,00,00,00
                    HEX        00,00,00,00,00,00,06,DD,DB,30,00,00,00,00,00,00,00,00,00,00,03,BD,DD,60,00,00,00,00,00,00
                    HEX        00,00,00,00,00,00,AD,DC,30,00,00,00,00,00,00,00,00,00,00,00,00,03,CD,DA,10,00,00,00,00,00
                    HEX        00,00,00,00,00,1C,DD,90,00,00,00,00,00,00,00,00,00,00,00,00,00,00,09,DD,C1,00,00,00,00,00
                    HEX        00,00,00,00,02,DD,D3,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,3D,DD,20,00,00,00,00
                    HEX        00,00,00,00,2D,DC,10,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,01,CD,D2,00,00,00,00
                    HEX        00,00,00,02,DD,B1,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,1B,DD,20,00,00,00
                    HEX        00,00,00,1C,DC,10,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,01,CD,C1,00,00,00
                    HEX        00,00,00,AD,D1,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,1C,DA,00,00,00
                    HEX        00,00,07,DD,30,00,00,00,00,00,00,00,00,17,AB,BB,BB,A7,00,00,00,00,00,00,00,02,DD,70,00,00
                    HEX        00,00,1D,D7,00,00,00,00,00,03,33,33,33,DD,DC,CC,CC,DD,C1,00,00,00,00,00,00,00,7D,D1,00,00
                    HEX        00,00,AD,B0,00,00,00,00,05,DD,DD,DD,DD,D3,00,00,00,04,DC,AA,AA,AA,A7,00,00,00,0B,DA,00,00
                    HEX        00,03,DD,20,00,00,00,00,7D,C5,44,44,4A,41,AC,CC,CC,A1,5D,CC,CC,CC,DD,C1,00,00,02,DD,30,00
                    HEX        00,0A,DA,00,00,03,33,34,DB,02,66,66,40,0A,DD,DD,DD,D9,02,00,00,00,04,DC,00,00,00,AD,A0,00
                    HEX        00,2D,D2,00,07,DD,DD,DD,D3,4D,DD,DD,DA,0B,DD,DD,DD,DA,02,BC,CC,CC,A1,5D,40,00,00,2D,D2,00
                    HEX        00,9D,A0,00,9D,B5,44,44,A0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0B,DD,DD,DD,D9,0D,90,00,00,0A,D9,00
                    HEX        00,DD,40,02,DA,03,66,64,00,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,0D,A0,00,00,04,DD,00
                    HEX        03,DD,00,07,D2,6D,DD,DD,A0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,0D,A0,00,00,00,DD,30
                    HEX        09,DA,00,09,D0,AD,DD,DD,D0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,0D,A0,00,00,00,AD,90
                    HEX        0B,D6,00,0A,D0,AD,DD,DD,D0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,0D,A0,00,00,00,6D,C0
                    HEX        0D,D2,00,0A,D0,AD,DD,DD,D0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,0D,A0,00,00,00,1D,D0
                    HEX        2D,D0,00,0A,D0,AD,DD,DD,D0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,0D,A0,00,00,00,0D,D2
                    HEX        4D,C0,00,0A,D0,AD,DD,DD,D0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,0D,A0,00,00,00,0B,D4
                    HEX        7D,A0,00,0A,D0,AD,DD,DD,D0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,0D,A0,00,00,00,0A,D7
                    HEX        8D,A0,00,0A,D0,AD,DD,DD,D0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,0D,D4,00,00,00,0A,D8
                    HEX        9D,90,00,0A,D0,AD,DD,DD,D0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,01,CD,70,00,00,09,D9
                    HEX        9D,90,00,0A,D0,AD,DD,DD,D0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,03,1B,D7,00,00,09,D9
                    HEX        8D,A0,00,0A,D0,AD,DD,DD,D0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,0D,40,BD,10,00,0A,D8
                    HEX        7D,A0,00,0A,D0,AD,DD,DD,D0,AD,DD,DD,DD,0B,DD,DD,DD,DA,0D,DD,DD,DD,DA,0D,D4,2D,70,00,0A,D7
                    HEX        6D,B0,00,09,D1,9D,DD,DD,D0,8D,DD,DD,DC,07,DD,DD,DD,D4,09,DD,DD,DD,D4,2D,DA,0D,A0,00,0B,D6
                    HEX        2D,D0,00,04,D5,1A,CC,CB,40,0A,BB,BB,A2,10,34,44,44,20,00,34,44,44,20,AD,D7,1D,80,00,0D,D3
                    HEX        1D,D1,00,00,CD,40,00,00,1B,70,00,00,03,CB,54,44,44,71,6A,64,44,44,5B,DD,D1,9D,20,00,1D,D1
                    HEX        0C,D4,00,00,1C,DD,CC,CC,DD,DD,DD,DD,DD,DD,DD,DD,DD,A0,CD,DD,DD,DD,DD,DD,90,DB,00,00,4D,C0
                    HEX        0A,D9,00,00,00,7A,BB,BA,92,6A,AA,AA,A8,10,23,33,AD,24,DD,DD,DD,DD,DD,DD,17,D4,00,00,9D,A0
                    HEX        04,DC,00,00,00,00,00,00,00,00,00,00,00,00,00,00,BC,0B,DD,DD,DD,DD,DD,DA,0C,C0,00,00,CD,40
                    HEX        00,DD,30,00,00,00,00,00,00,00,00,00,00,00,00,00,BC,0B,DD,DD,DD,DD,DD,D2,4D,50,00,03,DD,00
                    HEX        00,AD,A0,00,00,00,00,00,00,00,00,00,00,00,00,00,9D,22,CD,DD,DD,DD,DD,B0,BD,00,00,0A,DA,00
                    HEX        00,3D,D1,00,00,00,00,00,00,00,00,00,00,00,00,00,1D,C2,04,BD,DD,DD,DD,43,D8,00,00,1D,D3,00
                    HEX        00,0B,D9,00,00,00,00,00,00,00,00,00,00,00,00,00,04,DD,A3,02,AD,DD,DC,0A,D1,00,00,9D,C0,00
                    HEX        00,04,DD,10,00,00,00,00,00,00,00,00,00,00,00,00,00,19,DD,B6,01,9D,D3,1D,A0,00,01,DD,40,00
                    HEX        00,00,BD,A0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,18,DD,C7,10,01,BD,20,00,0A,DB,00,00
                    HEX        00,00,2D,D4,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,05,CD,DA,AD,D5,00,00,4D,D3,00,00
                    HEX        00,00,09,DD,10,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,04,AD,DA,30,00,01,DD,90,00,00
                    HEX        00,00,00,BD,B0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0B,DC,00,00,00
                    HEX        00,00,00,1D,DA,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,AD,D2,00,00,00
                    HEX        00,00,00,03,DD,A0,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,0A,DD,30,00,00,00
                    HEX        00,00,00,00,4D,DA,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,AD,D6,00,00,00,00
                    HEX        00,00,00,00,04,DD,B1,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,00,1B,DD,60,00,00,00,00
                    HEX        00,00,00,00,00,3D,DD,40,00,00,00,00,00,00,00,00,00,00,00,00,00,00,04,DD,D3,00,00,00,00,00
                    HEX        00,00,00,00,00,01,CD,DA,10,00,00,00,00,00,00,00,00,00,00,00,00,01,AD,DC,20,00,00,00,00,00
                    HEX        00,00,00,00,00,00,09,DD,D9,10,00,00,00,00,00,00,00,00,00,00,01,9D,DD,90,00,00,00,00,00,00
                    HEX        00,00,00,00,00,00,00,2B,DD,DA,30,00,00,00,00,00,00,00,00,03,AD,DD,B3,00,00,00,00,00,00,00
                    HEX        00,00,00,00,00,00,00,00,4B,DD,DC,94,10,00,00,00,00,01,49,CD,DD,C4,00,00,00,00,00,00,00,00
                    HEX        00,00,00,00,00,00,00,00,00,3A,DD,DD,DD,BA,99,99,AB,DD,DD,DD,A3,00,00,00,00,00,00,00,00,00
                    HEX        00,00,00,00,00,00,00,00,00,00,04,AC,DD,DD,DD,DD,DD,DD,CA,40,00,00,00,00,00,00,00,00,00,00
                    HEX        00,00,00,00,00,00,00,00,00,00,00,00,13,67,89,99,76,31,00,00,00,00,00,00,00,00,00,00,00,00

******************************************************************************************
